; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Main game logic

; Various game constants.
player_min_y        equ $18
player_bottom_y     equ player_min_y + 192
player_move_speed   equ 30
max_subpixel        equ 100
total_coins         equ 137
bottom_start_x      equ 148
bottom_start_y      equ player_bottom_y
bonus_object_index  equ 93
bonus_display_time  equ 240
bonus_tile          equ $2243
cash_bag_tile       equ $21a3
map_base_address    equ $2006
score_dollar        equ $2049
score_base          equ score_dollar + 1
bonus_base          equ score_dollar + 80
digit_base          equ 78
frame_length        equ 15
cop_frame_length    equ 4
lock_length         equ 120
cop_lock_length     equ 180
stage_end_delay     equ 240
bonus_timer_length  equ 240
lives_display       equ $23e0
stage_display       equ $239e
exit_timer_length   equ 10

; Coin action table constants.
coin_action_table   equ $1000
coin_action_bag     equ -1
coin_action_bonus   equ 1

; Color constants.
gate_color      equ $c8
door_color      equ $38
biff_color      equ $c9
buzz_color      equ door_color
bull_color      equ $89
pork_chop_color equ $1c

; Coin check tile positions.
coin_pos_center_bottom  equ 6
coin_pos_center_top     equ 10
coin_pos_center_right   equ 3
coin_pos_center_left    equ 5
coin_pos_top            equ 14
coin_pos_bottom         equ 2
coin_pos_left           equ 7
coin_pos_right          equ 1

; Directional bits.
direction_none  equ 0
direction_up    equ 1
direction_right equ 2
direction_down  equ 4
direction_left  equ 8

; Game phase constants.
phase_start_delay   equ 0
phase_enter_maze    equ 1
phase_normal        equ 2

; Timer durations.
start_delay_timer   equ 60

; Board edge constants.
left_board_edge     equ 72
right_board_edge    equ 176
wrap_tile_shift     equ 26

; Bonus constants.
bonus_left_x        equ left_board_edge + 48
bonus_right_x       equ bonus_left_x + 8
bonus_cash_y        equ $68
bonus_bonus_y       equ $88

; Door constants.
door_time_to_blink  equ 240
door_blink_time     equ 60
door_bit_vertical   equ 1
door_bit_blink      equ 2

; Door structure offsets.
door_address        equ 0
door_flags          equ 2
door_timer          equ 3
door_size           equ 4

; Cop behavior flags.
cop_choose_direction equ 128

; Alternating player map storage locations.
player_1_stage  equ $2500
player_2_stage  equ $2a00

; Score table offsets.
score_coin_offset   equ coin_score - score_table
score_hat_offset    equ hat_score - score_table
score_cash_offset   equ cash_score - score_table
score_cop_offset    equ cop_score - score_table

; Bonus table offsets.
bonus_hat_offset    equ hat_bonus - bonus_table
bonus_cash_offset   equ cash_bonus - bonus_table
bonus_cop_offset    equ cop_bonus - bonus_table

; Player structure offset.
player_x                equ 0
player_y                equ 1
player_subpixel         equ 2
player_tile_id          equ 3
player_tile_x           equ 4
player_tile_y           equ 5
player_direction        equ 6
player_animation        equ 7
player_frame_offset     equ 9
player_frame_timer      equ 10
player_page             equ 11
player_hpos_reg         equ 12
player_speed            equ 13
player_base_speed       equ 14
player_vert_door        equ 15
player_horiz_door       equ 17
player_door_1           equ 19
player_door_2           equ 23
player_last_trigger     equ 27
player_stop_direction   equ 28
player_lock_time        equ 29
player_size             equ 30

cop_flags               equ 15
cop_size                equ 16

; Green gate phases.
gate_spawn    equ 0
gate_close    equ 1
gate_hold     equ 2
gate_open     equ 3

; Green gate animation timer.
gate_frame_time equ 30
gate_hold_time  equ 60

; Green gate structure offsets.
gate_address    equ 0
gate_phase      equ 2
gate_timer      equ 3
gate_index      equ 4
gate_size       equ 5

; Data.
player_1_data       equ $20
cop_1_data          equ player_1_data + player_size
cop_2_data          equ cop_1_data + cop_size
cop_3_data          equ cop_2_data + cop_size
cop_4_data          equ cop_3_data + cop_size
game_timer          equ cop_4_data + cop_size
center_gate_left    equ game_timer + 1
center_gate_right   equ center_gate_left + gate_size
moving_gate_1       equ center_gate_right + gate_size
moving_gate_2       equ moving_gate_1 + gate_size
skip_gate           equ moving_gate_2 + gate_size
bonus_timer         equ skip_gate + 1   ; 16-bit timer.
cash_timer          equ bonus_timer + 2 ; 16-bit timer.
cop_lock_timer      equ cash_timer + 2
last_trigger        equ cop_lock_timer + 1
active_score        equ last_trigger + 2    ; 24-bit score.
next_cash_score     equ active_score + 3    ; Two of these.
next_bonus_score    equ next_cash_score + 2 ; Two of these.
cop_entry_timer_1   equ next_bonus_score + 2
cop_entry_timer_2   equ cop_entry_timer_1 + 1
cash_bonus_value    equ cop_entry_timer_2 + 1 ; Two of these.
bonus_bonus_value   equ cash_bonus_value + 2    ; Two of these.
remaining_coins     equ bonus_bonus_value + 2   ; Two of these.
stage_number        equ remaining_coins + 2 ; Two of these.
bonus_tile_id       equ stage_number + 2
cop_to_process      equ bonus_tile_id + 1
open_tunnel         equ cop_to_process + 1
exits_open          equ open_tunnel + 1
active_player       equ exits_open + 1
input_trigger       equ active_player + 1
active_stick        equ input_trigger + 1
random_value        equ active_stick + 1
script_time         equ random_value + 1
script_direction    equ script_time + 1
script_trigger      equ script_direction + 1
script_position     equ script_trigger + 1
next_coin_sound     equ script_position + 2
exit_timer          equ next_coin_sound + 1

; Non-page 0 data.
player_1_score          equ $300
player_2_score          equ $303
keypad_cooldown         equ $306
last_key_pressed        equ $307
player_1_lives          equ $308
player_2_lives          equ $309
player_1_extra          equ $30a
player_2_extra          equ $30b
player_color_pattern    equ $30c

; Macro for handling pause checks.
    mac check_pause
    lda     last_key_pressed
    cmp     #KEYPAUSE
    bne     .no_pause
    jsr     pause_loop
    jmp     {1}
.no_pause
    endm

start_game_from_intro subroutine
    jsr     start_transition
    lda     #<game_display_list
    sta     sDLISTL
    lda     #>game_display_list
    sta     sDLISTH
    mwa     game_dli,VDSLST
    mwa     game_vblank,VVBLKD
    lda     #$86
    sta     sCOLOR0
    lda     #$1e
    sta     sCOLOR1
    lda     #gate_color
    sta     sCOLOR2
    lda     #door_color
    sta     sCOLOR3
    jmp     start_new_life

start_game subroutine
    jsr     start_transition
    jsr     safe_clear_page_0
    jsr     clear_sprites
    inc     new_game
    lda     game_mode
    bmi     .attract_stage
    lda     start_stage
    jmp     .store_stage
.attract_stage
    lda     #0                      ; Attract mode uses stage 0.
.store_stage
    sta     stage_number
    sta     stage_number + 1
    lda     #-1                     ; Indicate a fresh stage for
    sta     remaining_coins + 1     ; player 2.
    lda     #3
    sta     $ff
    jsr     clear_page
    lda     #$20
    sta     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    lda     #<game_display_list
    sta     sDLISTL
    lda     #>game_display_list
    sta     sDLISTH
    mwa     game_dli,VDSLST
    mwa     game_vblank,VVBLKD
    lda     #2
    sta     player_1_lives
    sta     player_2_lives
    lda     #$a0
    sta     $fd
    lda     #$30
    sta     $ff
    ldx     #-1
    stx     cop_1_data + player_hpos_reg
    inx
    stx     player_1_data + player_hpos_reg
    stx     $fe
    stx     $fc
    inx
    stx     cop_2_data + player_hpos_reg
    inx
    stx     cop_3_data + player_hpos_reg
    inx
    stx     cop_4_data + player_hpos_reg
    jsr     copy_page
    inc     $ff
    inc     $fd
    jsr     copy_page
    inc     $ff
    inc     $fd
    jsr     copy_page
    inc     $ff
    inc     $fd
    jsr     copy_page
    lda     #$98
    sta     sCOLPM0
    lda     #$86
    sta     sCOLOR0
    lda     #$1e
    sta     sCOLOR1
    lda     #gate_color
    sta     sCOLOR2
    lda     #door_color
    sta     sCOLOR3
    lda     #coin_action_bag
    sta     coin_action_table + total_coins - 40
    sta     coin_action_table + total_coins - 70
    sta     coin_action_table + total_coins - 100
    sta     coin_action_table + total_coins - 130
    lda     #coin_action_bonus
    sta     coin_action_table + total_coins - 55
    sta     coin_action_table + total_coins - 85
    lda     #player_move_speed
    sta     player_1_data + player_base_speed
    asl
    sta     player_1_data + player_speed
    ldy     #$13
    sty     cop_1_data + player_page
    iny
    sty     player_1_data + player_page
    iny
    sty     cop_2_data + player_page
    iny
    sty     cop_3_data + player_page
    iny
    sty     cop_4_data + player_page
    ldy     #0
    sty     player_1_extra
    sty     player_2_extra
    dey
    sty     last_key_pressed
    mwa     player_1_stage,$fe
    mwa     map_layout,$fc
    jsr     copy_map_layout
    mwa     player_2_stage,$fe
    mwa     map_layout,$fc
    jsr     copy_map_layout
    lda     intermission
    beq     start_new_stage         ; Check intermission preview.
    ldx     #$30
    stx     CHBASE
    cmp     #1                      ; Figure out which intermission.
    beq     .preview_1
    cmp     #2
    beq     .preview_2
    jmp     start_intermission_3
.preview_1
    jmp     start_intermission_1
.preview_2
    jmp     start_intermission_2
    ; Fall through into the new stage code.

; Set up a new stage and fall into the game loop.
start_new_stage subroutine
    ldy     active_player
    lda     remaining_coins,y       ; Hacky check here for player 2's
    cmp     #-1                     ; first life.
    php
    lda     #total_coins
    sta     remaining_coins,y
    lda     #score_cash_offset
    sta     next_cash_score,y
    lda     #bonus_cash_offset
    sta     cash_bonus_value,y
    plp                             ; Check for player 2's first life.
    bne     start_new_life
    jmp     start_player_intro

; Ready a new life.
start_new_life subroutine
    lda     #$30                    ; This needs to be here because flow
    sta     CHBASE                  ; sometimes comes from the car intro.
    lda     #0                      ; Similarly, reconfigure audio.
    sta     AUDCTL
    ldy     active_player
    lda     #4
    sta     next_coin_sound
    lda     #1
    sta     random_value
    lda     game_mode
    bmi     .attract_mode
    cmp     #mode_two_player_two
    bne     .one_stick
    tya
    sta     active_stick
    ora     #4
    sta     CONSOL
    jmp     .one_stick
.attract_mode
    lda     script_number
    beq     .script_1
    cmp     #1
    beq     .script_2
    mwa     attract_script_3,script_position
    jmp     .script_ready
.script_2
    mwa     attract_script_2,script_position
    jmp     .script_ready
.script_1
    mwa     attract_script_1,script_position
.script_ready
    jsr     advance_script
    ldy     active_player
.one_stick
    ldx     stage_number,y
    txa
    and     #1
    sta     open_tunnel
    cpx     #max_bonus_object
    bcc     .bonus_id_ok
    ldx     #max_bonus_object - #1
.bonus_id_ok
    lda     bonus_score_map,x
    sta     next_bonus_score,y
    lda     bonus_display_map,x
    sta     bonus_bonus_value,y
    lda     red_cop_speeds,x
    sta     cop_1_data + player_base_speed
    lda     blue_cop_speeds,x
    sta     cop_3_data + player_base_speed
    lda     green_cop_speeds,x
    sta     cop_2_data + player_base_speed
    lda     yellow_cop_speeds,x
    sta     cop_4_data + player_base_speed
    txa
    asl
    tax
    lda     bonus_image_map,x
    sta     $fe
    lda     bonus_image_map + 1,x
    sta     $ff
    txa
    lsr
    tax
    ldy     #31
.bonus_image_loop
    lda     ($fe),y
    sta     [$3000 + tileset_length],y
    dey
    bpl     .bonus_image_loop
    jsr     copy_color_pattern
    ldy     active_player
    beq     .use_player_1_score
    lda     player_2_score
    sta     active_score
    lda     player_2_score + 1
    sta     active_score + 1
    lda     player_2_score + 2
    jmp     .score_ready
.use_player_1_score
    lda     player_1_score
    sta     active_score
    lda     player_1_score + 1
    sta     active_score + 1
    lda     player_1_score + 2
.score_ready
    sta     active_score + 2
    cpy     #0
    beq     .start_player_1
    mwa     player_2_stage,$fc
    jmp     .copy_stage
.start_player_1
    mwa     player_1_stage,$fc
.copy_stage
    mwa     map_base_address,$fe
    jsr     copy_map
    lda     #0                      ; Digit clean up.
    sta     score_base + 2
    sta     score_base + 3
    sta     score_base + 4
    sta     score_base + 5
    lda     #bonus_object_index
    ora     bonus_image_mask,x
    sta     bonus_tile_id
    tay
    sty     stage_display
    iny
    sty     stage_display + 1
    iny
    sty     stage_display + 40
    iny
    sty     stage_display + 41
    jsr     display_score
    ldy     active_player
    lda     player_1_lives,y
    clc
    adc     #78
    sta     lives_display
    ldy     #player_1_data
    jsr     erase_player
    ldy     #cop_1_data
    jsr     erase_player
    ldy     #cop_2_data
    jsr     erase_player
    ldy     #cop_3_data
    jsr     erase_player
    ldy     #cop_4_data
    jsr     erase_player
    lda     #0
    sta     $23d9                   ; Clear the entry doors.
    sta     $23da
    sta     $2035
    sta     $2036
    sta     $2195                   ; Clear the side doors.
    sta     $21bd
    sta     $2196
    sta     $21be
    sta     $21b1
    sta     $21d9
    sta     $2235
    sta     $225d
    sta     $2236
    sta     $225e
    sta     $2251
    sta     $2279
    sta     player_1_data + player_vert_door
    sta     player_1_data + player_vert_door + 1
    sta     player_1_data + player_horiz_door
    sta     player_1_data + player_horiz_door + 1
    sta     exits_open
    ldy     #last_trigger + 1
.clear_loop
    sta     $0,y
    cpy     #game_timer
    beq     .clear_loop_done
    dey
    jmp     .clear_loop
.clear_loop_done
    lda     #$21
    sta     center_gate_left + gate_address + 1
    sta     center_gate_right + gate_address + 1
    sta     moving_gate_1 + gate_address + 1
    sta     moving_gate_2 + gate_address + 1
    lda     #$f1
    sta     center_gate_left + gate_address
    lda     #$f5
    sta     center_gate_right + gate_address
    lda     #$e9
    sta     moving_gate_1 + gate_address
    lda     #$fd
    sta     moving_gate_2 + gate_address
    ldx     #-1
    stx     center_gate_left + gate_index
    stx     center_gate_right + gate_index
    inx
    stx     moving_gate_1 + gate_index
    ldx     #9
    stx     moving_gate_2 + gate_index
    lda     #moving_gate_1
    sta     skip_gate
    lda     #bottom_start_x
    sta     HPOSP0
    sta     player_1_data + player_x
    ldy     #bottom_start_y
    sty     player_1_data + player_y
    lda     #165
    sta     player_1_data + player_tile_id
    mva     #0,player_1_data + player_last_trigger
    lda     #start_delay_timer
    sta     game_timer
    jsr     hide_bonus_display
    sta     player_1_data + player_direction
    sta     player_1_data + player_frame_offset
    sta     cop_1_data + player_speed
    sta     cop_2_data + player_speed
    sta     cop_3_data + player_speed
    sta     cop_4_data + player_speed
    mwa     player_animation_up,player_1_data + player_animation
    lda     #frame_length
    sta     player_1_data + player_frame_timer
    lda     #1
    sta     last_trigger
    jsr     reset_cop_colors
    lda     #180
    sta     cop_entry_timer_1
    lda     #120
    sta     cop_entry_timer_2
    lda     #left_board_edge + 4
    sta     HPOSP1
    sta     HPOSM3
    sta     cop_1_data + player_x
    sta     cop_2_data + player_x
    lda     #left_board_edge + 6
    sta     HPOSM2
    lda     #left_board_edge + 8
    sta     HPOSM1
    lda     #left_board_edge + 10
    sta     HPOSM0
    lda     #right_board_edge - 4
    sta     HPOSP2
    sta     HPOSP3
    sta     cop_3_data + player_x
    sta     cop_4_data + player_x
    lda     #$68
    sta     cop_1_data + player_y
    sta     cop_3_data + player_y
    lda     #$88
    sta     cop_2_data + player_y
    sta     cop_4_data + player_y
    lda     #<cop_right_1
    sta     cop_1_data + player_animation
    sta     cop_2_data + player_animation
    lda     #>cop_right_1
    sta     cop_1_data + player_animation + 1
    sta     cop_2_data + player_animation + 1
    lda     #<cop_left_1
    sta     cop_3_data + player_animation
    sta     cop_4_data + player_animation
    lda     #>cop_left_1
    sta     cop_3_data + player_animation + 1
    sta     cop_4_data + player_animation + 1
    lda     #65
    sta     cop_1_data + player_tile_id
    lda     #77
    sta     cop_3_data + player_tile_id
    lda     #91
    sta     cop_2_data + player_tile_id
    lda     #103
    sta     cop_4_data + player_tile_id
    lda     #cop_choose_direction
    sta     cop_1_data + cop_flags
    sta     cop_2_data + cop_flags
    sta     cop_3_data + cop_flags
    sta     cop_4_data + cop_flags
    ldy     #player_1_data
    jsr     initialize_sprite_common
    ldy     #cop_1_data
    sty     cop_to_process
    jsr     initialize_sprite_common
    ldy     #cop_2_data
    jsr     initialize_sprite_common
    ldy     #cop_3_data
    jsr     initialize_sprite_common
    ldy     #cop_4_data
    jsr     initialize_sprite_common
    lda     #$ff
    sta     last_key_pressed
    jsr     complete_transition
    lda     game_mode               ; Disable sounds in attract mode.
    bpl     .not_attract
    jsr     STOPTUN
.not_attract
    lda     #3                      ; Start the background jingle.
    jsr     DOTUNE

; Start delay logic loop
start_delay_loop subroutine
    jsr     wait_for_vb
    lda     game_mode
    bpl     .not_attract
    jsr     check_for_attract_cancel
.not_attract
    check_pause start_delay_loop
    dec     game_timer
    bne     .phase_active
    lda     #direction_up
    sta     player_1_data + player_direction
    jmp     enter_maze_loop
.phase_active
    ldy     #player_1_data
    jsr     process_player_animation
    jsr     process_movement
    jmp     start_delay_loop

; Maze entry logic loop
enter_maze_loop subroutine
    jsr     wait_for_vb
    lda     game_mode
    bpl     .not_attract
    jsr     check_for_attract_cancel
.not_attract
    check_pause enter_maze_loop
    lda     player_1_data + player_y
    cmp     #bottom_start_y - #player_sprite_height
    beq     .next_phase
    ldy     #player_1_data
    jsr     process_movement
    jmp     enter_maze_loop
.next_phase
    lda     #1                      ; Start the gate timers.
    sta     center_gate_left + gate_timer
    sta     center_gate_right + gate_timer
    sta     moving_gate_1 + gate_timer
    sta     moving_gate_2 + gate_timer
    lda     cop_1_data + player_base_speed
    sta     cop_1_data + player_speed
    lda     #direction_right
    sta     cop_1_data + player_direction
    ldy     #cop_1_data
    jsr     move_player
    ldy     active_player
    lda     remaining_coins,y
    beq     .show_out_sign
    ldx     #47                     ; Lock the entry doors.
    stx     $23d9
    stx     $23da
    inx
    stx     $2035
    stx     $2036
    ldx     #3
    stx     $200d                   ; Far side of top escape door.
    stx     $200e
    jmp     .close_side_doors
.show_out_sign
    jsr     open_exits
.close_side_doors
    lda     #6                      ; Play the sound.
    jsr     DOTUNE
    ldx     #3                      ; Close the side doors.
    ldy     active_player
    lda     stage_number,y
    and     #1
    beq     .close_bottom
    stx     $2195
    stx     $21bd
    ldx     #26
    stx     $2196
    stx     $21be
    dex
    stx     $21b1
    stx     $21d9
    jmp     normal_loop
.close_bottom
    stx     $2235
    stx     $225d
    ldx     #26
    stx     $2236
    stx     $225e
    dex
    stx     $2251
    stx     $2279                   ; Fall through to the normal loop.

normal_loop subroutine
    jsr     wait_for_vb
    lda     game_mode
    bpl     .not_attract
    jsr     check_for_attract_cancel
.not_attract
    check_pause normal_loop
    lda     player_1_data + player_y
    cmp     #player_min_y + #player_sprite_height - 1
    beq     .stage_complete         ; Check for stage completion.
    cmp     #player_bottom_y - [#player_sprite_height - 1]
    bne     .stage_not_complete
.stage_complete
    lda     #0                      ; Remove the "OUT" signs.
    sta     $23d9
    sta     $23da
    sta     $2035
    sta     $2036
    jsr     CLEARTUN                ; Stop all sounds.
    jsr     make_cops_angry
    jmp     exit_maze_loop
.stage_not_complete
    ldy     #center_gate_left       ; Process the gates.
    jsr     process_gate
    ldy     #center_gate_right
    jsr     process_gate
    ldy     #moving_gate_1
    jsr     process_gate
    ldy     #moving_gate_2
    jsr     process_gate
    ldy     #player_1_data + #player_door_1
    jsr     process_door            ; Check the doors.
    ldy     #player_1_data + #player_door_2
    jsr     process_door
    lda     cash_bag_tile
    beq     .cash_bag_done
    ldx     player_1_data + player_x
    cpx     #bonus_right_x
    bcs     .no_cash_pickup
    cpx     #bonus_left_x
    bcc     .no_cash_pickup
    ldy     player_1_data + player_y
    cpy     #bonus_cash_y
    bne     .no_cash_pickup
    ldy     active_player
    ldx     cash_bonus_value,y
    jsr     display_bonus
    ldy     active_player
    inx
    inx
    stx     cash_bonus_value,y
    ldx     next_cash_score,y
    txa
    clc
    adc     #3
    sta     next_cash_score,y
    jsr     increase_score
    mwa     player_happy_1,player_1_data + player_animation
    mva     #frame_length,player_1_data + player_frame_timer
    lda     #cop_frame_length
    sta     cop_1_data + player_frame_timer
    sta     cop_2_data + player_frame_timer
    sta     cop_3_data + player_frame_timer
    sta     cop_4_data + player_frame_timer
    mva     #lock_length,player_1_data + player_lock_time
    mva     #cop_lock_length,cop_lock_timer
    jsr     make_cops_angry
    lda     #7
    jsr     KILLTUNE
    lda     #8
    jsr     DOTUNE
    jmp     .hide_cash_bag
.no_cash_pickup
    dec     cash_timer
    bne     .cash_bag_done
.hide_cash_bag
    jsr     hide_cash_bag
.cash_bag_done
    lda     bonus_tile
    beq     .bonus_done
    ldx     player_1_data + player_x
    cpx     #bonus_right_x
    bcs     .no_bonus_pickup
    cpx     #bonus_left_x
    bcc     .no_bonus_pickup
    ldy     player_1_data + player_y
    cpy     #bonus_bonus_y
    bne     .no_bonus_pickup
    ldy     active_player
    ldx     bonus_bonus_value,y
    jsr     display_bonus
    ldy     active_player
    ldx     next_bonus_score,y
    jsr     increase_score
    lda     #$10
    jsr     DOTUNE
    jmp     .hide_bonus
.no_bonus_pickup
    dec     bonus_timer
    bne     .bonus_done
.hide_bonus
    jsr     hide_bonus
.bonus_done
    lda     player_1_data + player_lock_time
    beq     .handle_player_1_input  ; Not in happy/caught state.
    dec     player_1_data + player_lock_time
    beq     .done_happy             ; Happy over, continue.
    jsr     draw_player
    jsr     color_happy_dance
    ldy     #player_1_data
    jsr     process_player_animation
    jsr     process_cop_lock
    jmp     normal_loop
.done_happy
    lda     #9                      ; Stop the jingle.
    jsr     KILLTUNE
    lda     #3                      ; Restart the background jingle.
    jsr     DOTUNE
.handle_player_1_input
    lda     exits_open              ; Check on door blink.
    beq     .exits_closed
    dec     exit_timer
    bne     .exits_closed
    jsr     show_out_sign
.exits_closed
    jsr     check_controller
    sta     $f0                     ; Save this value for later.
    ora     player_1_data + player_direction
    ora     player_1_data + player_stop_direction
    ldx     #0                      ; Reset the gate collisions.
    stx     player_1_data + player_stop_direction
    ldx     player_1_data + player_tile_y
    beq     .horizontal_move_ok
    and     ~[#direction_left | #direction_right]
.horizontal_move_ok
    ldx     player_1_data + player_tile_x
    beq     .vertical_move_ok
    and     ~[#direction_up | #direction_down]
.vertical_move_ok
    tax
    and     #direction_left
    beq     .check_right_move
    ldy     #player_1_data
    jsr     check_left_movement
    beq     .check_right_move
    txa
    and     ~#direction_left
    tax
.check_right_move
    txa
    and     #direction_right
    beq     .check_up_movement
    ldy     #player_1_data
    jsr     check_right_movement
    beq     .check_up_movement
    txa
    and     ~#direction_right
    tax
.check_up_movement
    txa
    and     #direction_up
    beq     .check_down_movement
    ldy     #player_1_data
    jsr     check_up_movement
    beq     .check_down_movement
    txa
    and     ~#direction_up
    tax
.check_down_movement
    txa
    and     #direction_down
    beq     .move_check_done
    ldy     #player_1_data
    jsr     check_down_movement
    beq     .move_check_done
    txa
    and     ~#direction_down
    tax
.move_check_done
    txa                             ; Check for move conflicts.
    beq     .conflicts_done         ; No legal move.
    cmp     #direction_up           ; Check for only one direction.
    beq     .conflicts_done
    cmp     #direction_right
    beq     .conflicts_done
    cmp     #direction_down
    beq     .conflicts_done
    cmp     #direction_left
    beq     .conflicts_done
    and     $f0                     ; Joystick has priority.
    tax
    and     [#direction_up | #direction_down]
    bne     .priority_set           ; Vertical has priority.
    txa
    and     [#direction_left | #direction_right]
.priority_set
    tax
.conflicts_done
    stx     player_1_data + player_direction
    jsr     process_movement
    jsr     check_cop_collision
    lda     input_trigger           ; Only react if the trigger
    cmp     last_trigger            ; has changed.
    beq     .no_door
    sta     last_trigger            ; Update last trigger.
    cmp     #0
    bne     .no_door
    ldy     #player_1_data
    jsr     try_lock_door
.no_door
    lda     game_timer              ; Handle the bonus display.
    beq     .bonus_timer_done
    dec     game_timer
    bne     .bonus_timer_done
    jsr     hide_bonus_display
.bonus_timer_done
    jmp     normal_loop

; Logic for the maze exit phase.
exit_maze_loop subroutine
    jsr     wait_for_vb
    check_pause exit_maze_loop
    jsr     animate_cops
    ldy     #player_1_data
    jsr     move_player
    jsr     draw_player
    lda     player_1_data + player_tile_y
    bne     exit_maze_loop
    mwa     player_happy_1,player_1_data + player_animation
    mva     #frame_length,player_1_data + player_frame_timer
    mva     #stage_end_delay,game_timer
    lda     #9
    jsr     DOTUNE
    ; Fall through to stage_end_loop

; Logic for stage conclusion.
stage_end_loop subroutine
    jsr     wait_for_vb
    check_pause stage_end_loop
    jsr     animate_cops
    ldy     #player_1_data
    jsr     process_player_animation
    jsr     draw_player
    jsr     color_happy_dance
    dec     game_timer
    bne     stage_end_loop
    lda     #9                      ; Stop the jingle.
    jsr     KILLTUNE
    jsr     clean_stage
    lda     active_player           ; Reset the layout.
    beq     .reset_player_1
    mwa     player_2_stage,$fe
    jmp     .refresh_stage
.reset_player_1
    mwa     player_1_stage,$fe
.refresh_stage
    mwa     map_layout,$fc
    jsr     copy_map_layout
    jsr     backup_score
    ldy     active_player
    ldx     stage_number,y
    inx
    bne     .no_overflow
    dex                             ; Stay at stage 255.
.no_overflow
    stx     stage_number,y
    cpx     #2
    beq     .intermission_1
    cpx     #5
    beq     .intermission_2
    cpx     #9
    beq     .intermission_3
    jmp     start_new_stage
.intermission_1
    jmp     start_intermission_1
.intermission_2
    jmp     start_intermission_2
.intermission_3
    jmp     start_intermission_3

death_loop subroutine
    lda     #60                     ; Hold for one second.
    sta     game_timer
.hold_loop
    jsr     wait_for_vb
    ldy     #player_1_data
    jsr     process_player_animation
    jsr     draw_player
    jsr     animate_cops
    dec     game_timer
    bne     .hold_loop
    lda     #$50
    sta     AUDCTL
    lda     #$c                     ; Start the death tune.
    jsr     DOTUNE
    lda     #$d
    jsr     DOTUNE
    lda     #$e
    jsr     DOTUNE
    lda     #0                      ; Hide all cops.
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM0
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    ldy     #player_sprite_height - 1
    ldx     player_1_data + player_y
.death_frame_loop
    lda     player_happy_1,y
    sta     [$1400 + #player_sprite_height - 1],x
    lda     active_player
    beq     .rob_color
    lda     banks_happy_pattern_1,y
    jmp     .apply_color
.rob_color
    lda     rob_happy_pattern_1,y
.apply_color
    sta     [$3e00 + #player_sprite_height - 1],x
    dex
    dey
    bpl     .death_frame_loop
    lda     #5                      ; Crumple 5 bytes.
    sta     $ff
    lda     #10                     ; Offset to delete.
    sta     $fe
    lda     #20
    sta     game_timer
.crumple_loop
    jsr     wait_for_vb
    dec     game_timer
    bne     .crumple_loop
    lda     $fe                     ; Get the offset to remove.
    tay
    clc
    adc     player_1_data + player_y
    tax
.row_loop
    lda     $1400,x
    sta     $1401,x
    lda     $3e00,x
    sta     $3e01,x
    dex
    dey
    bpl     .row_loop
    dec     $ff
    beq     .crumple_done
    inc     $fe
    lda     #8
    sta     game_timer
    jmp     .crumple_loop
.crumple_done
    lda     #30
    sta     game_timer
    lda     #20
    sta     $ff                     ; Rows to remove.
.collapse_loop
    jsr     wait_for_vb
    dec     game_timer
    bne     .collapse_loop
    ldy     $ff
    lda     player_1_data + player_y
    clc
    adc     #14
    tax
.hat_row_loop
    lda     $1400,x
    sta     $1401,x
    lda     $3e00,x
    sta     $3e01,x
    dex
    dey
    bpl     .hat_row_loop
    lda     #8
    sta     game_timer
    dec     $ff
    bne     .collapse_loop
    lda     #60
    sta     game_timer
.final_delay
    jsr     wait_for_vb
    dec     game_timer
    bne     .final_delay
    lda     game_mode
    bpl     .normal_game
    jmp     next_attract_script     ; Attract mode death.
.normal_game
    jsr     backup_score
    jsr     clean_stage
    ldx     active_player
    lda     player_1_lives,x
    sec
    sbc     #1
    sta     player_1_lives,x
    bmi     game_over
    mwa     map_base_address,$fc
    ldy     active_player           ; Back up the stage data.
    beq     .backup_player_1
    lda     #>player_2_stage
    jmp     .backup_stage
.backup_player_1
    lda     #>player_1_stage
.backup_stage
    sta     $ff
    lda     #0
    sta     $fe
    jsr     copy_map
    lda     game_mode
    beq     .one_player
    bmi     .end_attract
    lda     active_player           ; Does the other player have lives?
    eor     #1
    tay
    lda     player_1_lives,y
    bmi     .one_player             ; No, proceed as if single player.
    sty     active_player           ; Switch to the other player.
    lda     remaining_coins,y       ; Is this player 2's first life?
    cmp     #-1
    bne     .one_player             ; Probably a misnamed label.
    jsr     start_transition
    jmp     start_new_stage         ; Need some additional initialization.
.one_player
    jmp     start_player_intro
.end_attract
    jmp     start_title_screen

; Game over handling.  X will have tha active player number.
game_over subroutine
    lda     game_mode
    beq     .true_game_over         ; Single player, really done.
    txa
    eor     #1
    tay
    lda     player_1_lives,y        ; Does the other player have lives?
    bmi     .true_game_over
    sty     active_player           ; Switch to the other player.
    jmp     start_game_over
.true_game_over
    ldy     #2                      ; Copy scores to long term storage.
.score_loop
    lda     player_1_score,y
    sta     last_p1_score,y
    lda     player_2_score,y
    sta     last_p2_score,y
    dey
    bpl     .score_loop
    ldy     #2                      ; Find the higher score.
.find_high_loop
    lda     player_1_score,y
    cmp     player_2_score,y
    bcc     .player_2_high
    bne     .player_1_high
    dey
    bpl     .find_high_loop         ; Fall through on equal scores.
.player_1_high
    lda     #<player_1_score
    jmp     .compare_to_high
.player_2_high
    lda     #<player_2_score
.compare_to_high
    sta     $fe
    lda     #3
    sta     $ff
    ldy     #2
.compare_high_loop
    lda     high_score,y
    cmp     ($fe),y
    bcc     .new_high_score
    dey
    bpl     .compare_high_loop
    jmp     start_game_over         ; No high score.
.new_high_score
    ldy     #2
.update_high_score
    lda     ($fe),y
    sta     high_score,y
    dey
    bpl     .update_high_score
    jmp     start_game_over
    
; Check for user input that will abort attract mode.
check_for_attract_cancel subroutine
    lda     last_key_pressed
    bpl     .cancel
    lda     TRIG0
    beq     .cancel
    lda     SKSTAT
    and     #8
    beq     .cancel
    rts
.cancel
    ldx     #$ff                    ; Clear the stack.
    txs
    ; Fall through to attract advance.

; Move to the next attract script before
; returning to the title screen.
next_attract_script subroutine
    ldx     script_number
    cpx     #2
    bne     .next_script
    ldx     #-1
.next_script
    inx
    stx     script_number
    jmp     start_title_screen

; Clean up bonus objects, doors and gates.
clean_stage subroutine
    jsr     hide_cash_bag
    jsr     hide_bonus
    jsr     hide_bonus_display
    ldy     #center_gate_left
    jsr     erase_gate
    ldy     #center_gate_right
    jsr     erase_gate
    ldy     #moving_gate_1
    jsr     erase_gate
    ldy     #moving_gate_2
    jsr     erase_gate
    ldy     #player_1_data + #player_door_1
    jsr     erase_door
    ldy     #player_1_data + #player_door_2
    jmp     erase_door

; Pause loop.
pause_loop subroutine
    lda     #$ff
    sta     last_key_pressed
    jsr     STOPTUN                 ; Halt sounds.
.loop
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #KEYPAUSE
    beq     .unpause
    cmp     #KEYRESET
    beq     .reset
    cmp     #KEYSTART
    bne     .loop
    ldx     #$ff
    txs
    inx
    stx     active_player
    dec     new_game
    jmp     start_player_intro
.reset
    ldx     #$ff
    txs
    jmp     start_title_screen
.unpause
    lda     #$ff
    sta     last_key_pressed
    jsr     STARTTUN                ; Resume sounds.
    rts

; Process sprite movement.
process_movement subroutine
    ldy     #player_1_data
    jsr     move_player
    jsr     draw_player
    lda     cop_lock_timer          ; See if the cops are locked.
    beq     .no_cop_lock
    jmp     process_cop_lock
.no_cop_lock
    lda     cop_entry_timer_2       ; If the last timer is expired,
    beq     .cop_entry_done         ; all cops have entered.
    lda     cop_entry_timer_1
    beq     .process_cop_timer_2
    dec     cop_entry_timer_1
    bne     .cop_entry_done
    lda     cop_3_data + player_base_speed
    sta     cop_3_data + player_speed
    lda     #direction_left
    sta     cop_3_data + player_direction
    ldy     #cop_3_data
    jsr     move_player
    jmp     .cop_entry_done
.process_cop_timer_2
    dec     cop_entry_timer_2
    bne     .cop_entry_done
    lda     cop_2_data + player_base_speed
    sta     cop_2_data + player_speed
    lda     cop_4_data + player_base_speed
    sta     cop_4_data + player_speed
    lda     #direction_right
    sta     cop_2_data + player_direction
    lda     #direction_left
    sta     cop_4_data + player_direction
    ldy     #cop_2_data
    jsr     move_player
    ldy     #cop_4_data
    jsr     move_player
.cop_entry_done
    ldy     cop_to_process
    cpy     #cop_1_data
    beq     .first_cop_pair
    lda     #cop_1_data
    sta     cop_to_process
    jsr     process_cop
    ldy     #cop_4_data
    jmp     process_cop
.first_cop_pair
    lda     #cop_3_data
    sta     cop_to_process
    jsr     process_cop
    ldy     #cop_2_data
    jmp     process_cop

; Process cop lock animation.
process_cop_lock
    dec     cop_lock_timer
    beq     .end_cop_lock
    jmp     animate_cops
.end_cop_lock
    ; Fall through to reset colors.

; Set the cops to their distinct colors.
reset_cop_colors subroutine
    lda     #biff_color
    sta     sCOLPM1
    lda     #bull_color
    sta     sCOLPM2
    lda     #pork_chop_color
    sta     sCOLPM3
    rts

; Process animations for all four cops.
animate_cops subroutine
    ldy     cop_to_process
    cpy     #cop_4_data
    beq     .loop_to_cop_1
    tya
    clc
    adc     #cop_size
    sta     cop_to_process
    jmp     .update_cop_to_process
.loop_to_cop_1
    lda     #cop_1_data
.update_cop_to_process
    sta     cop_to_process
    jsr     process_player_animation
    jmp     draw_cop

; Put all the cops into angry mode.
make_cops_angry subroutine
    lda     #<cop_angry_1
    sta     cop_1_data + player_animation
    sta     cop_2_data + player_animation
    sta     cop_3_data + player_animation
    sta     cop_4_data + player_animation
    lda     #>cop_angry_1
    sta     cop_1_data + player_animation + 1
    sta     cop_2_data + player_animation + 1
    sta     cop_3_data + player_animation + 1
    sta     cop_4_data + player_animation + 1
    lda     #door_color
    sta     sCOLPM1
    sta     sCOLPM2
    sta     sCOLPM3
    rts

; Specific cop routines.  Returns the movement
; flags in the accumulator.
process_red_cop subroutine
process_blue_cop subroutine
process_green_cop subroutine
    jsr     player_centered_on_tile
    bne     process_cop_no_move_change
    jsr     has_line_of_sight
    bmi     .no_los
    pha
    jsr     get_valid_cop_directions
    sta     $ff
    pla
    and     $ff
    beq     .no_los
    jmp     process_cop_resume
.no_los
    jsr     get_valid_cop_directions
    jsr     get_random_cop_direction
    jmp     process_cop_resume

; Perform cop behavior for the cop
; whose pointer is in y.
process_cop subroutine
    sty     $f0                     ; Save the pointer.
    lda     cop_flags,y             ; Check for a new direction.
    bpl     process_cop_no_move_change
    and     ~[#cop_choose_direction]
    sta     cop_flags,y             ; Clear the flag.
    cpy     #cop_1_data
    bne     .not_red
    jmp     process_red_cop
.not_red
    cpy     #cop_2_data
    bne     .not_green
    jmp     process_green_cop
.not_green
    cpy     #cop_3_data
    bne     process_yellow_cop
    jmp     process_blue_cop
process_yellow_cop subroutine
    jsr     player_centered_on_tile
    bne     process_cop_no_move_change
    jsr     get_valid_cop_directions
    jsr     get_random_cop_direction
    ; Fall through to processing.
process_cop_resume
    ldy     $f0                     ; Recover the pointer.
    sta     player_direction,y
    cmp     #0
    beq     .skip_move
    jmp     .finish_move
process_cop_no_move_change
    lda     player_direction,y      ; Check for door hits.
    cmp     #direction_up
    beq     .door_check_up
    cmp     #direction_down
    beq     .door_check_down
    cmp     #direction_left
    beq     .door_check_left
    cmp     #direction_right
    bne     .finish_move
    jsr     check_door_hit_right
    bne     .flip_horizontal_move
    jmp     .finish_move
.door_check_up
    jsr     check_door_hit_up
    beq     .finish_move
.flip_vertical_move
    lda     player_direction,y
    eor     #direction_up | #direction_down
    jmp     .store_direction_flip
.door_check_down
    jsr     check_door_hit_down
    bne     .flip_vertical_move
    jmp     .finish_move
.door_check_left
    jsr     check_door_hit_left
    beq     .finish_move
.flip_horizontal_move
    lda     player_direction,y
    eor     #direction_left | #direction_right
.store_direction_flip
    sta     player_direction,y
.finish_move
    jsr     move_player
    jmp     draw_cop
.skip_move
    jsr     process_player_animation
    ; Fall through to drawing code.

; Draw the cop whose address is in y.
draw_cop subroutine
    lda     player_animation,y
    clc
    adc     player_frame_offset,y
    sta     $fc
    lda     player_animation + 1,y
    adc     #0
    sta     $fd
    lda     player_y,y
    sta     $fe
    sta     $fa
    lda     #$3e
    sta     $fb
    lda     player_page,y
    sta     $ff
    ldy     #player_sprite_height + 1
    lda     #0
    sta     ($fe),y                 ; Erase two bytes in each direction.
    dey
    sta     ($fe),y
    dey
.loop
    lda     ($fc),y
    sta     ($fe),y
    dey
    bpl     .loop
    lda     #0
    dec     $fe
    iny
    sta     ($fe),y                 ; Erase two more bytes.
    dec     $fe
    sta     ($fe),y
    rts

; Check to see if a player is on an even tile boundary.
; Expects the player pointer in y.  Clears the zero bit
; the player is on such a bounary.
player_centered_on_tile subroutine
    lda     player_tile_x,y
    ora     player_tile_y,y
    rts

; Check for player collision with cops.
check_cop_collision subroutine
    ldy     #cop_1_data
.loop
    lda     player_speed,y
    beq     .next                   ; Non-moving, ignore.
    lda     cop_lock_timer
    bne     .next                   ; In lock, ignore.
    lda     player_x,y
    sec
    sbc     player_1_data + player_x
    jsr     absolute_value
    cmp     #4
    bcs     .next                   ; No collision.
    lda     player_y,y
    sec
    sbc     player_1_data + player_y
    jsr     absolute_value
    cmp     #8
    bcs     .next                   ; No collision.
    jsr     CLEARTUN                ; Stop all sounds.
    ldx     #$ff                    ; Reset the stack.
    txs
    jmp     death_loop
.next
    cpy     #cop_4_data
    beq     .done
    tya
    clc
    adc     #cop_size
    tay
    jmp     .loop
.done
    rts

; Get the directions a cop can move in.
; Expects the cop pointer in y, will disregard
; the direction opposite the current movement.
; Results will be in the accumulator.  Assumes
; cop pointer is backed up in $f0.
get_valid_cop_directions subroutine
    lda     #0
    sta     $f1                     ; Running result.
    jsr     check_up_movement
    bne     .no_up
    lda     #direction_up
    sta     $f1
.no_up
    ldy     $f0                     ; Recover the pointer.
    jsr     check_down_movement
    bne     .no_down
    lda     #direction_down
    ora     $f1
    sta     $f1
.no_down
    ldy     $f0                     ; Recover the pointer.
    jsr     check_left_movement
    bne     .no_left
    lda     #direction_left
    ora     $f1
    sta     $f1
.no_left
    ldy     $f0                     ; Recover the pointer.
    jsr     check_right_movement
    bne     .no_right
    lda     #direction_right
    ora     $f1
    sta     $f1
.no_right
    ldy     $f0                     ; Recover the pointer.
    lda     player_direction,y      ; Filter out the previous direction.
    cmp     #direction_down
    bmi     .left_shift
    lsr                             ; Two right shifts invert the flag.
    lsr
    jmp     .filter_direction
.left_shift
    asl                             ; Two left shifts invert the flag.
    asl
.filter_direction
    eor     #$ff                    ; Flip the mask.
    and     $f1                     ; Strip the direction.
    beq     .override               ; In the blocked side tunnel dead
    rts                             ; ends, sometimes you can only go
.override                           ; one direction.
    lda     $f1                     ; Use the only eligible direction.
    rts

; Get a random direction of movement from the
; potential directions in the accumulator.
; Returns the selected direction in the accumulator.
get_random_cop_direction subroutine
    sta     $f2                     ; Save the direction.
.get_direction
    lda     random_value            ; Get a random number.
    asl                             ; Apply this algorithm I found somewhere.
    bcc     .random_done
    eor     #$1d
.random_done
    sta     random_value
    and     #$03                    ; Mask to two bits.
    tax
    lda     #1                      ; Shift a random number of times.
.loop
    dex
    bmi     .loop_out
    asl
    jmp     .loop
.loop_out
    and     $f2                     ; Is this direction OK?
    beq     .get_direction          ; No, try again.
    rts

; Check to see if the cop whose pointer is in y
; has a line of sight on the player.  If so,
; returns the direction of the sight line in
; the accumulator, -1 if no line.
has_line_of_sight subroutine
    lda     player_x,y
    cmp     player_1_data + player_x
    bcs     .player_to_left
    ldx     #direction_right
    lda     player_1_data + player_x
    sec
    sbc     player_x,y
    jmp     .store_x_delta
.player_to_left
    ldx     #direction_left
    sbc     player_1_data + player_x; Carry is set.
.store_x_delta
    sta     $f1                     ; Save the x delta.
    stx     $f2                     ; Save the x direction.
    lda     player_tile_id,y        ; Set up the cop's tile.
    sta     $ff
    lda     player_y,y
    cmp     player_1_data + player_y
    bcs     .player_to_top
    ldx     #direction_down
    lda     player_1_data + player_y
    sec
    sbc     player_y,y
    jmp     .store_y_delta
.player_to_top
    ldx     #direction_up
    sbc     player_1_data + player_y; Carry is set.
.store_y_delta
    sta     $f3                     ; Save the y delta.
    stx     $f4                     ; Save the y direction.
    cmp     #16                     ; Check for horizontal LOS.
    bcs     .no_horizontal_los
    lda     $f1                     ; No horizontal if x delta is 0.
    bne     .horizontal_los
.no_horizontal_los
    lda     $f3                     ; No vertical if y delta is 0.
    beq     .no_los
    lda     $f1
    cmp     #8                      ; Check for vertical LOS.
    bcs     .no_los
    lda     #13                     ; Tile delta.
    sta     $fd
    lda     $f3                     ; Divide delta by 16 to get a tile count.
    lsr
    lsr
    lsr
    lsr
    tax
    lda     $f4                     ; Check the direction.
    sta     $fc
    cmp     #direction_up
    beq     .check_by_subtract
    jmp     .check_by_add
.horizontal_los
    lda     #1                      ; Tile delta.
    sta     $fd
    lda     $f1                     ; Divide delta by 8 to get a tile count.
    lsr
    lsr
    lsr
    tax
    lda     $f2                     ; Check the direction.
    sta     $fc
    cmp     #direction_left
    beq     .check_by_subtract
.check_by_add
    sty     $fb                     ; Save the pointer.
    clc
    ldy     $ff
.los_add_loop
    lda     map_collision,y
    bne     .los_fail
    dex
    bmi     .has_los
    tya
    adc     $fd
    tay
    jmp     .los_add_loop
.check_by_subtract
    sty     $fb                     ; Save the pointer.
    sec
    ldy     $ff
.los_subtract_loop
    lda     map_collision,y
    bne     .los_fail
    dex
    bmi     .has_los
    tya
    sbc     $fd
    tay
    jmp     .los_subtract_loop
.has_los
    ldy     $fb
    lda     $fc
    rts
.los_fail
    ldy     $fb                     ; Recover the pointer.
.no_los
    lda     #-1
    rts

; Validate a proposed movement location.  Expects
; the tile to check in $fe-$ff, the direction of movement
; in the accumulator, and the tile subpixel
; coordinates in x and y.  Sets the carry bit if the
; move is legal, clears it otherwise.
validate_move subroutine
    cmp     #direction_right
    beq     .check_right_move
    cmp     #direction_left
    beq     .check_left_move
    cmp     #direction_down
    beq     .check_down_move
    cpy     #15
    beq     .check_up_tile
    sec
    rts
.check_up_tile
    tya
    pha
    ldy     #0                      ; Check the current tile.
    jmp     .check_tile
.check_right_move
    cpx     #1
    beq     .check_right_tile
    sec
    rts
.check_right_tile
    tya
    pha
    ldy     #2                      ; Check tile to the right.
.check_tile
    lda     ($fe),y
    beq     .move_ok
    cmp     #33                     ; Check for wall tile.
    bpl     .move_ok
    pla
    tay
    clc                             
    rts
.move_ok
    pla
    tay
    sec
    rts
.check_left_move
    cpx     #7
    beq     .check_up_tile          ; Same code as up.
    sec
    rts
.check_down_move
    cpy     #1
    beq     .check_down_tile
    sec
    rts
.check_down_tile
    tya
    pha
    ldy     #80                     ; Check one tile down.
    jmp     .check_tile

; Move the player whose pointer is in y.
move_player subroutine
    sty     $fd                     ; Save the pointer.
    ldx     player_direction,y
    bne     .move_requested
    jmp     .no_move
.move_requested
    lda     #0                      ; Animation change flag.
    sta     $f0
    txa                             ; Get the direction.
    and     #direction_left | #direction_up
    bne     .negative_move
    lda     player_subpixel,y
    clc
    adc     player_speed,y
    cmp     #max_subpixel
    bcs     .try_move
    jmp     .update_subpixel
.try_move
    ldy     #1                      ; Number of pixels to move.
    sec
.positive_count_loop
    sbc     #max_subpixel
    cmp     #max_subpixel
    bcc     .attempt_move
    iny
    jmp     .positive_count_loop
.negative_move
    lda     player_subpixel,y
    sec
    sbc     player_speed,y
    bcc     .try_negative_move
    jmp     .update_subpixel
.try_negative_move
    ldy     #1                      ; Number of pixels to move.
    clc
.negative_count_loop
    adc     #max_subpixel
    bcs     .attempt_move
    iny
    jmp     .negative_count_loop
.attempt_move
    sty     $fc                     ; Save the pixel count.
    ldy     $fd                     ; Recover the pointer.
    sta     player_subpixel,y
    txa                             ; Get the direction back.
    cmp     #direction_up
    bne     .not_up
    jmp     .try_move_up
.not_up
    cmp     #direction_down
    bne     .not_down
    jmp     .try_move_down
.not_down
    cmp     #direction_left
    bne     .try_move_right
    jmp     .try_move_left
.try_move_right
    lda     player_x,y              ; Check the wrap zone.
    cmp     #right_board_edge
    beq     .wrap_to_left
    cmp     #left_board_edge + 4
    bcc     .next_right_no_tile
    cmp     #right_board_edge - 4
    bcs     .next_right_no_tile
    jmp     .no_wrap_zone_right
.wrap_to_left
    lda     #left_board_edge
    sta     player_x,y
    lda     #0
    sta     player_tile_x,y
    lda     player_tile_id,y
    sec
    sbc     #12
    sta     player_tile_id,y
    jmp     .next_right_wrap
.no_wrap_zone_right
    jsr     check_right_movement
    bne     .finish_move_right
    ldx     player_tile_x,y
    cpx     #7
    beq     .new_tile_right
    inx
    txa
.next_right
    sta     player_tile_x,y
    cmp     #5
    bne     .next_right_no_tile
    jsr     check_door_trigger_right
.next_right_no_tile
    ldx     player_x,y
    inx
    stx     player_x,y
.next_right_wrap
    cpy     #cop_1_data
    bcs     .no_coin_check_right
    lda     player_tile_x,y
    cmp     #coin_pos_center_left
    bne     .no_center_coin_right
    ldx     player_tile_id,y
    inx
    jsr     prepare_tile
    jsr     check_center_coin
    jmp     .end_coin_check_right
.no_center_coin_right
    cmp     #coin_pos_right
    bne     .no_coin_check_right
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_left_right_coin
.end_coin_check_right
    ldy     $fd
.no_coin_check_right
    lda     #1                      ; Set the animation flag.
    sta     $f0
    dec     $fc                     ; Check the pixel count.
    beq     .finish_move_right
    jmp     .try_move_right
.new_tile_right
    ldx     player_tile_id,y
    cpx     #77                     ; Check for wrap.
    beq     .check_right_wrap_high
    cpx     #103
    bne     .no_right_wrap
    lda     open_tunnel
    beq     .finish_move_right
    jmp     .next_right_no_tile
.check_right_wrap_high
    lda     open_tunnel
    beq     .next_right_no_tile
    jmp     .finish_move_right
.no_right_wrap
    inx
    stx     player_tile_id,y
    lda     #0
    jmp     .next_right
.finish_move_right
    lda     player_base_speed,y     ; Set the speed.
    sta     player_speed,y
    cpy     #cop_1_data             ; Set the AI flag.
    bcc     .no_cop_flag_right
    lda     #cop_choose_direction
    sta     cop_flags,y
.no_cop_flag_right
    lda     $f0                     ; Check the animation flag.
    beq     .no_animation_change_right
    cpy     #cop_1_data
    bcc     .player_animation_right
    lda     #<cop_animation_right
    sta     player_animation,y
    lda     #>cop_animation_right
    jmp     .finish_animation_right
.player_animation_right
    lda     #<player_animation_right
    sta     player_animation,y
    lda     #>player_animation_right
.finish_animation_right
    sta     player_animation + 1,y
.no_animation_change_right
    ldx     player_x,y
    jsr     update_player_hpos
    jmp     process_player_animation
.try_move_up
    ldy     $fd
.try_move_up_loop
    jsr     check_up_movement
    bne     .finish_move_up
    ldx     player_tile_y,y
    beq     .new_tile_up
    dex
    txa
.next_up
    sta     player_tile_y,y
    cmp     #7
    bne     .no_up_trigger_check
    jsr     check_door_trigger_up
.no_up_trigger_check
    ldx     player_y,y
    dex
    stx     player_y,y
    cpy     #cop_1_data
    bcs     .no_coin_check_up
    lda     player_tile_y,y
    cmp     #coin_pos_center_bottom
    bne     .no_center_coin_up
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_center_coin
    jmp     .end_coin_check_up
.no_center_coin_up
    cmp     #coin_pos_top
    bne     .no_coin_check_up
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_top_bottom_coin
.end_coin_check_up
    ldy     $fd
.no_coin_check_up
    lda     #1                      ; Set the animation flag.
    sta     $f0
    dec     $fc                     ; Check the pixel count.
    beq     .finish_move_up
    jmp     .try_move_up_loop
.new_tile_up
    lda     player_tile_id,y
    sec
    sbc     #13
    sta     player_tile_id,y
    lda     #15
    jmp     .next_up
.finish_move_up
    lda     player_base_speed,y     ; Set the speed.
    asl                             ; Double speed on vertical.
    sta     player_speed,y
    cpy     #cop_1_data             ; Set the AI flag.
    bcc     .no_cop_flag_up
    lda     #cop_choose_direction
    sta     cop_flags,y
.no_cop_flag_up
    lda     $f0                     ; Check the animation flag.
    beq     .no_animation_change_up
    cpy     #cop_1_data
    bcc     .player_animation_up
    lda     #<cop_animation_up
    sta     player_animation,y
    lda     #>cop_animation_up
    jmp     .finish_animation_up
.player_animation_up
    lda     #<player_animation_up
    sta     player_animation,y
    lda     #>player_animation_up
.finish_animation_up
    sta     player_animation + 1,y
.no_animation_change_up
    jmp     process_player_animation
.try_move_down
    ldy     $fd
.try_move_down_loop
    jsr     check_down_movement
    bne     .finish_move_down
    ldx     player_tile_y,y
    cpx     #15
    beq     .new_tile_down
    inx
    txa
.next_down
    sta     player_tile_y,y
    cmp     #9
    bne     .no_down_trigger_check
    jsr     check_door_trigger_down
.no_down_trigger_check
    ldx     player_y,y
    inx
    stx     player_y,y
    cpy     #cop_1_data
    bcs     .no_coin_check_down
    lda     player_tile_y,y
    cmp     #coin_pos_center_top
    bne     .no_center_coin_down
    lda     player_tile_id,y
    clc
    adc     #13
    tax
    jsr     prepare_tile
    jsr     check_center_coin
    jmp     .end_coin_check_down
.no_center_coin_down
    cmp     #coin_pos_bottom
    bne     .no_coin_check_down
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_top_bottom_coin
.end_coin_check_down
    ldy     $fd
.no_coin_check_down
    lda     #1                      ; Set the animation flag.
    sta     $f0
    dec     $fc                     ; Check the pixel count.
    beq     .finish_move_down
    jmp     .try_move_down_loop
.new_tile_down
    lda     player_tile_id,y
    clc
    adc     #13
    sta     player_tile_id,y
    lda     #0
    jmp     .next_down
.finish_move_down
    lda     player_base_speed,y     ; Set the speed.
    asl                             ; Double speed on vertical.
    sta     player_speed,y
    cpy     #cop_1_data             ; Set the AI flag.
    bcc     .no_cop_flag_down
    lda     #cop_choose_direction
    sta     cop_flags,y
.no_cop_flag_down
    lda     $f0                     ; Check the animation flag.
    beq     .no_animation_change_down
    cpy     #cop_1_data
    bcc     .player_animation_down
    lda     #<cop_animation_down
    sta     player_animation,y
    lda     #>cop_animation_down
    jmp     .finish_animation_down
.player_animation_down
    lda     #<player_animation_down
    sta     player_animation,y
    lda     #>player_animation_down
.finish_animation_down
    sta     player_animation + 1,y
.no_animation_change_down
    jmp     process_player_animation
.try_move_left
    lda     player_x,y              ; Check the wrap zone.
    cmp     #left_board_edge
    beq     .wrap_to_right
    cmp     #left_board_edge + 4
    bcc     .next_left_no_tile
    cmp     #right_board_edge - 3
    bcs     .next_left_no_tile
    jmp     .no_wrap_zone_left
.wrap_to_right
    lda     #right_board_edge
    sta     player_x,y
    lda     #0
    sta     player_tile_x,y
    lda     player_tile_id,y
    clc
    adc     #12
    sta     player_tile_id,y
    jmp     .next_left_wrap
.no_wrap_zone_left
    jsr     check_left_movement
    bne     .finish_move_left
    ldx     player_tile_x,y
    beq     .new_tile_left
    dex
    txa
.next_left
    sta     player_tile_x,y
    cmp     #3
    bne     .next_left_no_tile
    jsr     check_door_trigger_left
.next_left_no_tile
    ldx     player_x,y
    dex
    stx     player_x,y
.next_left_wrap
    cpy     #cop_1_data
    bcs     .no_coin_check_left
    lda     player_tile_x,y
    cmp     #coin_pos_center_right
    bne     .no_center_coin_left
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_center_coin
    jmp     .end_coin_check_left
.no_center_coin_left
    cmp     #coin_pos_left
    bne     .no_coin_check_left
    ldx     player_tile_id,y
    jsr     prepare_tile
    jsr     check_left_right_coin
.end_coin_check_left
    ldy     $fd
.no_coin_check_left
    lda     #1                      ; Set the animation flag.
    sta     $f0
    dec     $fc                     ; Check the pixel count.
    beq     .finish_move_left
    jmp     .try_move_left
.new_tile_left
    ldx     player_tile_id,y
    cpx     #65                     ; Check for wrap.
    beq     .check_left_wrap_high
    cpx     #91
    bne     .no_left_wrap
    lda     open_tunnel
    beq     .finish_move_left
    jmp     .next_left_no_tile
.check_left_wrap_high
    lda     open_tunnel
    beq     .next_left_no_tile
    jmp     .finish_move_left
.no_left_wrap
    dex
    stx     player_tile_id,y
    lda     #7
    jmp     .next_left
.finish_move_left
    lda     player_base_speed,y     ; Set the speed.
    sta     player_speed,y
    cpy     #cop_1_data             ; Set the AI flag.
    bcc     .no_cop_flag_left
    lda     #cop_choose_direction
    sta     cop_flags,y
.no_cop_flag_left
    lda     $f0                     ; Check the animation flag.
    beq     .no_animation_change_left
    cpy     #cop_1_data
    bcc     .player_animation_left
    lda     #<cop_animation_left
    sta     player_animation,y
    lda     #>cop_animation_left
    jmp     .finish_animation_left
.player_animation_left
    lda     #<player_animation_left
    sta     player_animation,y
    lda     #>player_animation_left
.finish_animation_left
    sta     player_animation + 1,y
.no_animation_change_left
    ldx     player_x,y
    jsr     update_player_hpos
    jmp     process_player_animation
.update_subpixel
    sta     player_subpixel,y
    jmp     process_player_animation
.no_move
    rts

; Erases the player whose pointer is in y.
erase_player subroutine
    lda     player_page,y
    sta     $ff
    lda     player_y,y
    sta     $fe
    ldy     #player_sprite_height
    lda     #0
.loop
    sta     ($fe),y
    dey
    bpl     .loop
    rts

; Update a player HPOS register.  Expects the player's
; data pointer in y and the new HPOS in x.
; Does nothing for the actual players, since that's
; handled in DLIs.
update_player_hpos subroutine
    sty     $fd                     ; Save the pointer.
    lda     player_hpos_reg,y
    bmi     .missiles
    tay
    txa
    sta     HPOSP0,y
    jmp     .done
.missiles
    txa
    clc
    sta     HPOSM3
    adc     #2
    sta     HPOSM2
    adc     #2
    sta     HPOSM1
    adc     #2
    sta     HPOSM0
.done
    ldy     $fd                     ; Recover the pointer.
    rts

; Initialize common data shared by the players and cops.
; Expects the block pointer in y.
initialize_sprite_common subroutine
    lda     #0
    sta     player_subpixel,y
    sta     player_tile_x,y
    sta     player_tile_y,y
    sta     player_frame_offset,y
    lda     #frame_length
    sta     player_frame_timer,y
    rts

; Handle animation logic for the player whose
; address is in y.
process_player_animation subroutine
    ldx     player_frame_timer,y
    dex
    bne     .update_timer
    lda     player_frame_offset,y
    beq     .offset_to_16
    lda     #0
    jmp     .store_offset
.offset_to_16
    lda     #16
.store_offset
    sta     player_frame_offset,y
    cpy     #cop_1_data
    bcc     .player_frame
    ldx     #cop_frame_length
    jmp     .update_timer
.player_frame
    ldx     #frame_length
.update_timer
    stx     player_frame_timer,y
    rts

; Draw the player, either Rob or Banks.
draw_player subroutine
    lda     player_1_data + player_animation
    clc
    adc     player_1_data + player_frame_offset
    sta     $fc
    lda     player_1_data + player_animation + 1
    adc     #0
    sta     $fd
    lda     player_1_data + player_y
    sta     $fe
    sta     $fa
    lda     #$3e
    sta     $fb
    lda     #$14
    sta     $ff
    ldy     #player_sprite_height
    lda     #0
    sta     ($fe),y
    dey
.loop
    lda     ($fc),y
    sta     ($fe),y
    lda     player_1_data + player_lock_time
    bne     .no_color_update
    lda     player_color_pattern,y
    sta     ($fa),y
.no_color_update
    dey
    bpl     .loop
    iny
    dec     $fe
    tya
    sta     ($fe),y
    rts

; Update the color pattern for the current
; happy dance frame.
color_happy_dance subroutine
    lda     player_1_data + player_frame_offset
    beq     .happy_color_1
    lda     active_player
    beq     .rob_happy_color_2
    mwa     banks_happy_pattern_2,$fc
    jmp     .apply_happy_color
.rob_happy_color_2
    mwa     rob_happy_pattern_2,$fc
    jmp     .apply_happy_color
.happy_color_1
    lda     active_player
    beq     .rob_happy_color_1
    mwa     banks_happy_pattern_1,$fc
    jmp     .apply_happy_color
.rob_happy_color_1
    mwa     rob_happy_pattern_1,$fc
.apply_happy_color
    lda     #$3e
    sta     $ff
    lda     player_1_data + player_y
    sta     $fe
    ldy     #15
.happy_color_loop
    lda     ($fc),y
    sta     ($fe),y
    dey
    bpl     .happy_color_loop
    rts

; Copy a map layout into the video RAM.  Used for
; the initial layout for each player.  Expects the layout
; source in $fc-$fd and the destination in $fe-$ff.
copy_map_layout subroutine
    ldx     #map_height
.row_loop
    ldy     #map_width - 1
.column_loop
    lda     ($fc),y
    sta     ($fe),y
    dey
    bpl     .column_loop
    dex
    beq     .done
    lda     $fc
    clc
    adc     #map_width
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    lda     $fe
    clc
    adc     #40
    sta     $fe
    bcc     .row_loop
    inc     $ff
    jmp     .row_loop
.done
    rts

; Transfer a player's map layout.  Can either go
; from video RAM to the player's storage area or
; vice-versa.  Expects the source in $fc-$fd and
; the target in $fe-$ff.
copy_map subroutine
    jsr     copy_page
    inc     $fd
    inc     $ff
    jsr     copy_page
    inc     $fd
    inc     $ff
    jsr     copy_page
    inc     $fd
    inc     $ff
    jmp     copy_page

; Get the next input from the attract mode script.
advance_script subroutine
    ldy     #0
    lda     (script_position),y
    sta     script_time
    iny
    lda     (script_position),y
    sta     script_direction
    iny
    lda     (script_position),y
    sta     script_trigger
    lda     script_position
    clc
    adc     #3
    sta     script_position
    bcc     .no_carry
    inc     script_position + 1
.no_carry
    rts

; Check input.  Checks the active stick if in an actual
; game, processes the attract mode script otherwise.
check_controller subroutine
    lda     game_mode
    bpl     .check_sticks
    lda     script_trigger
    sta     input_trigger
    lda     script_direction
    dec     script_time
    bne     .done
    tax
    jsr     advance_script
    txa
    rts
.check_sticks
    ldy     active_stick
    ldx     TRIG0,y
    stx     input_trigger
    tya                             ; Shift for POT lookup.
    asl
    tay
    lda     #0                      ; Assume no input.
    ldx     sPOT0,y
    cpx     #170
    bcs     .right
    cpx     #50
    bcs     .check_vpot
.left
    ora     #direction_left
    jmp     .check_vpot
.right
    ora     #direction_right
.check_vpot
    ldx     sPOT1,y
    cpx     #170
    bcs     .down
    cpx     #50
    bcs     .done
.up
    ora     #direction_up
    rts
.down
    ora     #direction_down
.done
    rts

; Attempt to lock a door.
try_lock_door subroutine
    lda     player_1_data + player_door_1 + door_address + 1
    beq     .use_door_1
    lda     player_1_data + player_door_2 + door_address + 1
    beq     .door_available
.done
    lda     #$a                     ; Play the invalid sound.
    jsr     DOTUNE
    rts                             ; Both doors in use.
.door_available
    tya
    clc
    adc     #player_door_2 + #door_address
    tax
    jmp     .lock_door
.use_door_1
    tya
    clc
    adc     #player_door_1 + #door_address
    tax
.lock_door
    lda     #door_time_to_blink     ; Door pointer is in x.
    sta     door_timer,x
    lda     player_1_data + player_vert_door + 1
    bne     .lock_vertical_door
    lda     player_1_data + player_horiz_door + 1
    beq     .done                   ; No junction passed yet.
    sta     $ff
    sta     door_address + 1,x
    lda     player_1_data + player_horiz_door
    sta     $fe
    sta     door_address,x
    jsr     check_player_obstruction
    bcc     .no_horizontal_obstruction
    lda     #0
    sta     door_address + 1,x      ; Disable the door.
    rts
.no_horizontal_obstruction
    lda     #0
    sta     door_flags,x
    sta     player_1_data + player_horiz_door + 1
    tay
    lda     #53 | #128
    sta     ($fe),y
    iny
    sta     ($fe),y
    ldy     #40
    lda     #59 | #128
    sta     ($fe),y
    iny
    sta     ($fe),y
    jmp     .check_cop_trap
.lock_vertical_door
    sta     $ff
    sta     door_address + 1,x
    lda     player_1_data + player_vert_door
    sta     $fe
    sta     door_address,x
    jsr     check_player_obstruction
    bcc     .no_vertical_obstruction
    lda     #0
    sta     door_address + 1,x      ; Disable the door.
    rts
.no_vertical_obstruction
    lda     #door_bit_vertical
    sta     door_flags,x
    lda     #0
    sta     player_1_data + player_vert_door + 1
    tay
    lda     #75 | #128
    sta     ($fe),y
    ldy     #40
    sta     ($fe),y
    iny
    lda     #76 | #128
    sta     ($fe),y
    ldy     #1
    sta     ($fe),y
.check_cop_trap
    lda     #6                      ; Play the sound.
    jsr     DOTUNE
    ldx     #0                      ; Trapped cop count.
    lda     $ff
    cmp     #$20
    beq     .check_20_doors
    cmp     #$21
    beq     .check_21_doors
    cmp     #$22
    beq     .check_22_doors
    lda     $fe                     ; 23 doors.
    cmp     #$7b
    beq     .check_bottom_left
    cmp     #$8b
    beq     .check_bottom_right
    rts
.check_20_doors
    lda     $fe
    cmp     #$6b
    beq     .check_top_right
    rts
.check_21_doors
    lda     $fe
    cmp     #$0b
    beq     .check_top_right
.check_22_doors
    lda     $fe
    cmp     #$89
    beq     .check_bottom_left
    cmp     #$9d
    beq     .check_bottom_right
    rts
.check_top_right
    lda     $206b
    cmp     #51
    bcs     .continue_top_right
    jmp     .cop_trap_done
.continue_top_right
    lda     $210b
    cmp     #51
    bcs     .check_top_right_cops
    jmp     .cop_trap_done          ; Right of 160, up of 73.
.check_top_right_cops
    ldy     #cop_1_data
.top_right_loop
    lda     #160
    cmp     player_x,y
    bcs     .next_top_right
    lda     player_y,y
    cmp     #73
    bcs     .next_top_right
    inx                             ; Got a cop.
.next_top_right
    cpy     #cop_4_data
    beq     .show_cop_trap_score
    tya
    clc
    adc     #cop_size
    tay
    jmp     .top_right_loop
.check_bottom_left
    lda     $2289
    cmp     #51
    bcc     .cop_trap_done
    lda     $237b
    cmp     #51
    bcc     .cop_trap_done          ; Left of 88, down of 160.
    ldy     #cop_1_data
.bottom_left_loop
    lda     player_x,y
    cmp     #88
    bcs     .next_bottom_left
    lda     #160
    cmp     player_y,y
    bcs     .next_bottom_left
    inx                             ; Got a cop.
.next_bottom_left
    cpy     #cop_4_data
    beq     .show_cop_trap_score
    tya
    clc
    adc     #cop_size
    tay
    jmp     .bottom_left_loop
.check_bottom_right                 ; Check the two lower right doors.
    lda     $238b                   ; Slightly redundant, since one has
    cmp     #51                     ; already been checked, but code is
    bcc     .cop_trap_done          ; simpler and smaller this way.
    lda     $229d
    cmp     #51
    bcc     .cop_trap_done          ; Right of 160, down of 160.
    ldy     #cop_1_data
.bottom_right_loop
    lda     #160
    cmp     player_x,y
    bcs     .next_bottom_right
    lda     #160
    cmp     player_y,y
    bcs     .next_bottom_right
    inx                             ; Got a cop.
.next_bottom_right
    cpy     #cop_4_data
    beq     .show_cop_trap_score
    tya
    clc
    adc     #cop_size
    tay
    jmp     .bottom_right_loop
.show_cop_trap_score
    dex                             ; Convert to a table index.
    bmi     .cop_trap_done          ; No cops caught.
    stx     $ff                     ; Save the base index.
    txa
    asl                             ; Double the value.
    adc     #bonus_cop_offset       ; Carry is clear.
    tax
    jsr     display_bonus
    lda     #$11                    ; Play the sound effect.
    jsr     DOTUNE
    lda     $ff                     ; Triple the index.
    asl
    adc     $ff                     ; Carry is clear.
    adc     #score_cop_offset
    tax
    jmp     increase_score
.cop_trap_done
    rts

; Handle all the necessary logic for the
; door in y.
process_door subroutine
    lda     door_address + 1,y      ; Is the door active?
    beq     .done
    sta     $ff
    lda     door_address,y
    sta     $fe
    ldx     door_timer,y
    dex
    beq     .timer_expired
    stx     door_timer,y
    txa
    and     #3                      ; Blink every four frames.
    beq     .done
    ldx     door_flags,y
    txa
    and     #door_bit_blink
    bne     .blink_door
.done
    rts
.timer_expired
    ldx     door_flags,y
    txa
    and     #door_bit_blink
    beq     .start_blink
    lda     #0
    sta     door_address + 1,y      ; Disable the door.
    ldy     #0
    sta     ($fe),y                 ; Clear the tiles.
    iny
    sta     ($fe),y
    ldy     #40
    sta     ($fe),y
    iny
    sta     ($fe),y
    rts
.start_blink
    txa
    ora     #door_bit_blink
    sta     door_flags,y
    lda     #door_blink_time
    sta     door_timer,y
    rts
.blink_door
    txa
    and     #door_bit_vertical
    beq     .blink_horizontal
    ldy     #0
    lda     ($fe),y
    beq     .vertical_pattern_1
    tya
    sta     ($fe),y
    ldy     #41
    sta     ($fe),y
    lda     #75 | #128
    dey
    sta     ($fe),y
    lda     #76 | #128
    ldy     #1
    sta     ($fe),y
    rts
.vertical_pattern_1
    lda     #75 | #128
    sta     ($fe),y
    lda     #76 | #128
    ldy     #41
    sta     ($fe),y
    lda     #0
    dey
    sta     ($fe),y
    ldy     #1
    sta     ($fe),y
    rts
.blink_horizontal
    tay
    lda     ($fe),y
    beq     .horizontal_pattern_1
    tya
    sta     ($fe),y
    ldy     #41
    sta     ($fe),y
    lda     #59 | #128
    dey
    sta     ($fe),y
    lda     #53 | #128
    ldy     #1
    sta     ($fe),y
    rts
.horizontal_pattern_1
    lda     #53 | #128
    sta     ($fe),y
    ldy     #41
    lda     #59 | #128
    sta     ($fe),y
    lda     #0
    dey
    sta     ($fe),y
    ldy     #1
    sta     ($fe),y
    rts

; Checks left movement against the maze structure.
; Expects the player's data block in the y register.
; Sets the 0 bit if the move is legal.  Sets the
; negative bit on door hit.
check_left_movement subroutine
    jsr     check_door_hit_left
    bne     .hit_door
    lda     player_tile_x,y         ; If at x 0, do a tile check.
    beq     .check_tile
    lda     #0                      ; Set 0 bit.
    rts
.hit_door
    cpy     #cop_1_data             ; Player or cop?
    bcs     .cop_door_hit
    lda     #direction_left
    sta     player_stop_direction,y
.cop_door_hit
    lda     #-1
    rts
.check_tile
    stx     $ff
    ldx     player_tile_id,y
    cpx     #65                     ; Do wrap checks.
    beq     .check_wrap_high
    cpx     #91
    bne     .no_wrap
    ldx     $ff
    lda     open_tunnel
    cmp     #1
    rts
.check_wrap_high
    ldx     $ff
    lda     open_tunnel
    rts
.no_wrap
    lda     map_collision - 1,x
    ldx     $ff
    cmp     #0
    rts

; The same for the other directions.
check_right_movement subroutine
    jsr     check_door_hit_right
    bne     .hit_door
    lda     player_tile_x,y         ; If at x 0, do a tile check.
    beq     .check_tile
    lda     #0                      ; Set 0 bit.
    rts
.hit_door
    cpy     #cop_1_data             ; Player or cop?
    bcs     .cop_door_hit
    lda     #direction_right        ; Clears the 0 bit.
    sta     player_stop_direction,y
.cop_door_hit
    lda     #-1
    rts
.check_tile
    stx     $ff
    ldx     player_tile_id,y        ; Do wrap checks.
    cpx     #77
    beq     .check_wrap_high
    cpx     #103
    bne     .no_wrap
    ldx     $ff
    lda     open_tunnel
    cmp     #1
    rts
.check_wrap_high
    ldx     $ff
    lda     open_tunnel
    rts
.no_wrap
    ldx     player_tile_id,y
    lda     map_collision + 1,x
    ldx     $ff
    cmp     #0
    rts

check_up_movement subroutine
    jsr     check_door_hit_up
    bne     .hit_door
    lda     player_tile_y,y
    beq     .check_tile
    lda     #0                      ; Set the 0 bit.
    rts
.hit_door
    cpy     #cop_1_data             ; Player or cop?
    bcs     .cop_door_hit
    lda     #direction_up           ; Clears the 0 bit.
    sta     player_stop_direction,y
.cop_door_hit
    lda     #-1
    rts
.check_tile
    stx     $ff
    ldx     player_tile_id,y
    cpy     #cop_1_data             ; Special case for exits.
    bcs     .exit_check_done
    lda     exits_open              ; Are exits open?
    beq     .exit_check_done        ; Nope.
    cpx     #16                     ; Exit tile.
    bne     .exit_check_done
    ldx     $ff
    lda     #0
    rts
.exit_check_done
    lda     map_collision - 13,x
    ldx     $ff
    cmp     #0
    rts

check_down_movement subroutine
    jsr     check_door_hit_down
    bne     .hit_door
    lda     player_tile_y,y
    beq     .check_tile
    lda     #0                      ; Set the 0 bit.
    rts
.hit_door
    cpy     #cop_1_data             ; Player or cop?
    bcs     .cop_door_hit
    lda     #direction_down         ; Clears the 0 bit.
    sta     player_stop_direction,y
.cop_door_hit
    lda     #-1
    rts
.check_tile
    stx     $ff
    ldx     player_tile_id,y
    cpy     #cop_1_data             ; Special case for exits.
    bcs     .exit_check_done
    lda     exits_open              ; Are exits open?
    beq     .exit_check_done        ; Nope.
    cpx     #152                    ; Exit tile.
    bne     .exit_check_done
    ldx     $ff
    lda     #0
    rts
.exit_check_done
    lda     map_collision + 13,x
    ldx     $ff
    cmp     #0
    rts
    
; Check for door collisions for players moving left.
; Expects the player pointer in y, returns 0 if a door
; was not hit.
check_door_hit_left subroutine
    lda     player_tile_x,y         ; Check on xpos 5.
    cmp     #5
    bne     .no_hit
    sty     $fd                     ; Save the player pointer.
    lda     player_tile_id,y        ; Check for a door.
    tay
    lda     tile_map_low_byte,y
    sta     $fe
    lda     tile_map_high_byte,y
    sta     $ff
    ldy     #1
    lda     #76 | #128
    cmp     ($fe),y                 ; Check both high and low in
    beq     .hit_door               ; case the door is phasing out.
    ldy     #41
    cmp     ($fe),y
    beq     .hit_door
    ldy     $fd                     ; Recover the player pointer.
.no_hit
    lda     #0                      ; Set 0 bit.
    rts
.hit_door
    ldy     $fd                     ; Recover the player pointer.
    rts                             ; 0 bit not set.

; The same, but for other directions.
check_door_hit_right subroutine
    lda     player_tile_x,y         ; Check on xpos 3.
    cmp     #3
    bne     .no_hit
    sty     $fd                     ; Save the player pointer.
    lda     player_tile_id,y        ; Check for a door.
    tay
    lda     tile_map_low_byte,y
    sta     $fe
    lda     tile_map_high_byte,y
    sta     $ff
    ldy     #2
    lda     #75 | #128
    cmp     ($fe),y                 ; Check both high and low in
    beq     .hit_door               ; case the is phasing out.
    ldy     #42
    cmp     ($fe),y
    beq     .hit_door
    ldy     $fd                     ; Recover the player pointer.
.no_hit
    lda     #0                      ; Set 0 bit.
    rts
.hit_door
    ldy     $fd                     ; Recover the player pointer.
    rts                             ; 0 bit not set.

check_door_hit_up subroutine
    lda     player_tile_y,y         ; Check on ypos 9.
    cmp     #9
    bne     vertical_door_no_hit
    sty     $fd                     ; Save the player pointer.
    lda     player_tile_id,y        ; Check for a door.
    tay
    lda     tile_map_low_byte,y
    sta     $fe
    lda     tile_map_high_byte,y
    sta     $ff
    ldy     #0
    jmp     check_vertical_door_common

check_door_hit_down subroutine
    lda     player_tile_y,y         ; Check on ypos 7.
    cmp     #7
    bne     vertical_door_no_hit
    sty     $fd                     ; Save the player pointer.
    lda     player_tile_id,y        ; Check for a door.
    tay
    lda     tile_map_low_byte,y
    sta     $fe
    lda     tile_map_high_byte,y
    sta     $ff
    ldy     #80
    ; Fall through to common code.

check_vertical_door_common subroutine
    lda     ($fe),y
    and     #$7f                    ; Mask off the color bit.
    cmp     #52
    beq     .hit_door
    cmp     #53
    beq     .hit_door
    cmp     #64
    beq     .hit_door
    cmp     #65
    beq     .hit_door
    iny
    lda     ($fe),y
    cmp     #53 | #128
    beq     .hit_door
    ldy     $fd                     ; Recover the player pointer.
vertical_door_no_hit
    lda     #0                      ; Set 0 bit.
    rts
.hit_door
    ldy     $fd                     ; Recover the player pointer.
    rts                             ; 0 bit not set.

; Place the address corresponding
; to the tile ID in x into $fe-$ff.  This
; needs to happen for various collision
; checks.
prepare_tile subroutine
    lda     tile_map_low_byte,x
    sta     $fe
    lda     tile_map_high_byte,x
    sta     $ff
    rts

; Check to see if the player has reached a
; trigger point for a door.  Expects
; the player's data pointer in y.
check_door_trigger_right subroutine
    cpy     #player_1_data          ; Don't do this for cops.
    bne     .done
    lda     player_1_data + player_tile_id
    sec
    sbc     #13
    tax
    lda     tile_map_door_code,x
    cmp     #1
    beq     .on_trigger
    cmp     #2
    bne     .done
    dex
    cpx     player_1_data + player_last_trigger
    bne     .done
    ldx     player_1_data + player_tile_id
    lda     tile_map_low_byte,x
    sta     player_1_data + player_vert_door
    lda     tile_map_high_byte,x
    sta     player_1_data + player_vert_door + 1
    lda     #0
    sta     player_1_data + player_horiz_door + 1
    rts
.on_trigger
    stx     player_1_data + player_last_trigger
.done
    rts

check_door_trigger_up subroutine
    cpy     #player_1_data          ; Don't do this for cops.
    bne     .done
    ldx     player_1_data + player_tile_id
    lda     tile_map_door_code,x    ; No 13 subtraction because we want
    cmp     #1                      ; the next tile down.
    beq     .on_trigger
    cmp     #3
    bne     .done
    txa
    clc
    adc     #13
    cmp     player_1_data + player_last_trigger
    bne     .done
    tax
    lda     tile_map_low_byte,x
    sta     player_1_data + player_horiz_door
    lda     tile_map_high_byte,x
    sta     player_1_data + player_horiz_door + 1
    lda     #0
    sta     player_1_data + player_vert_door + 1
    rts
.on_trigger
    stx     player_1_data + player_last_trigger
.done
    rts

check_door_trigger_down subroutine
    cpy     #player_1_data          ; Don't do this for cops.
    bne     .done
    lda     player_1_data + player_tile_id
    sec
    sbc     #13
    tax
    lda     tile_map_door_code,x
    cmp     #1
    beq     .on_trigger
    cmp     #3
    bne     .done
    txa
    sec
    sbc     #13
    cmp     player_1_data + player_last_trigger
    bne     .done
    ldx     player_1_data + player_tile_id
    lda     tile_map_low_byte,x
    sta     player_1_data + player_horiz_door
    lda     tile_map_high_byte,x
    sta     player_1_data + player_horiz_door + 1
    lda     #0
    sta     player_1_data + player_vert_door + 1
    rts
.on_trigger
    stx     player_1_data + player_last_trigger
.done
    rts

check_door_trigger_left subroutine
    cpy     #player_1_data          ; Don't do this for cops.
    bne     .done
    lda     player_1_data + player_tile_id
    sec
    sbc     #12                     ; Interested in tile to the right.
    tax
    lda     tile_map_door_code,x
    cmp     #1
    beq     .on_trigger
    cmp     #2
    bne     .done
    inx
    cpx     player_1_data + player_last_trigger
    bne     .done
    ldx     player_1_data + player_tile_id
    inx
    lda     tile_map_low_byte,x
    sta     player_1_data + player_vert_door
    lda     tile_map_high_byte,x
    sta     player_1_data + player_vert_door + 1
    lda     #0
    sta     player_1_data + player_horiz_door + 1
    rts
.on_trigger
    stx     player_1_data + player_last_trigger
.done
    rts

    
; Check for center coin collision and remove it
; if necessary.  Expects the tile in question to
; be in $fe-$ff.
check_center_coin subroutine
    ldy     #0
    lda     ($fe),y
    cmp     #36
    beq     .remove_36
    cmp     #37
    beq     .remove_37_top
    cmp     #38
    beq     .remove_38_left
    cmp     #39
    beq     .remove_39
    rts
.remove_36
    lda     #0
    jmp     .top_right
.remove_37_top
    lda     #33
    jmp     .top_right
.remove_38_left
    lda     #34
    jmp     .top_right
.remove_39
    lda     #35
.top_right
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #34
    beq     .remove_34
    cmp     #38
    beq     .remove_38_right
    cmp     #42
    beq     .remove_42_top
    lda     #41                     ; Tile 43.
    jmp     .bottom_left
.remove_34
    lda     #0
    jmp     .bottom_left
.remove_38_right
    lda     #36
    jmp     .bottom_left
.remove_42_top
    lda     #40
.bottom_left
    sta     ($fe),y
    ldy     #40
    lda     ($fe),y
    cmp     #33
    beq     .remove_33
    cmp     #37
    beq     .remove_37_bottom
    cmp     #44
    beq     .remove_44_left
    lda     #41                     ; Tile 45.
    jmp     .bottom_right
.remove_33
    lda     #0
    jmp     .bottom_right
.remove_37_bottom
    lda     #36
    jmp     .bottom_right
.remove_44_left
    lda     #40
.bottom_right
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #40
    beq     .remove_40
    cmp     #42
    beq     .remove_42_bottom
    cmp     #44
    beq     .remove_44_right
    lda     #35                     ; Tile 46.
    jmp     .done
.remove_40
    lda     #0
    jmp     .done
.remove_42_bottom
    lda     #34
    jmp     .done
.remove_44_right
    lda     #33
.done
    sta     ($fe),y
    jmp     process_remove_coin

; Other coin checkers.
check_top_bottom_coin subroutine
    ldy     #40
    lda     ($fe),y
    cmp     #36
    beq     .remove_36
    cmp     #37
    beq     .remove_37_bottom
    cmp     #41
    beq     .remove_41_bottom
    cmp     #45
    beq     .remove_45
    cmp     #63
    bmi     .no_coin
    cmp     #75
    bmi     .bottom_left_coin
.no_coin
    rts
.bottom_left_coin
    sec
    sbc     #12
    jmp     .bottom_right
.remove_36
    lda     #0
    jmp     .bottom_right
.remove_37_bottom
    lda     #33
    jmp     .bottom_right
.remove_41_bottom
    lda     #40
    jmp     .bottom_right
.remove_45
    lda     #44
.bottom_right
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #34
    beq     .remove_34
    cmp     #35
    beq     .remove_35_bottom
    cmp     #42
    beq     .remove_42_bottom
    cmp     #46
    beq     .remove_46
    sec                             ; Coin with gate/door.
    sbc     #12
    jmp     .top_left
.remove_34
    lda     #0
    jmp     .top_left
.remove_35_bottom
    lda     #33
    jmp     .top_left
.remove_42_bottom
    lda     #40
    jmp     .top_left
.remove_46
    lda     #44
.top_left
    sta     ($fe),y
    ldy     #80
    lda     ($fe),y
    cmp     #33
    beq     .remove_33
    cmp     #35
    beq     .remove_35_top
    cmp     #37
    beq     .remove_37_top
    cmp     #39
    beq     .remove_39
    sec                             ; Coin with gate/door.
    sbc     #12
    jmp     .top_right
.remove_33
    lda     #0
    jmp     .top_right
.remove_35_top
    lda     #34
    jmp     .top_right
.remove_37_top
    lda     #36
    jmp     .top_right
.remove_39
    lda     #38
.top_right
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #40
    beq     .remove_40
    cmp     #41
    beq     .remove_41_top
    cmp     #42
    beq     .remove_42_top
    cmp     #43
    beq     .remove_43
    sec                             ; Coin with gate/door.
    sbc     #12
    jmp     .done
.remove_40
    lda     #0
    jmp     .done
.remove_41_top
    lda     #36
    jmp     .done
.remove_42_top
    lda     #34
    jmp     .done
.remove_43
    lda     #38
.done
    sta     ($fe),y
    jmp     process_remove_coin

check_left_right_coin subroutine
    ldy     #1
    lda     ($fe),y
    cmp     #36
    beq     .remove_36
    cmp     #38
    beq     .remove_38_left
    cmp     #41
    beq     .remove_41_left
    cmp     #43
    beq     .remove_43
    rts
.remove_36
    lda     #0
    jmp     .top_left
.remove_38_left
    lda     #34
    jmp     .top_left
.remove_41_left
    lda     #40
    jmp     .top_left
.remove_43
    lda     #42
.top_left
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #34
    beq     .remove_34
    cmp     #35
    beq     .remove_35_right
    cmp     #38
    beq     .remove_38_right
    lda     #37                     ; Tile 39.
    jmp     .bottom_right
.remove_34
    lda     #0
    jmp     .bottom_right
.remove_35_right
    lda     #33
    jmp     .bottom_right
.remove_38_right
    lda     #36
.bottom_right
    sta     ($fe),y
    ldy     #41
    lda     ($fe),y
    cmp     #33
    beq     .remove_33
    cmp     #35
    beq     .remove_35_left
    cmp     #44
    beq     .remove_44_left
    lda     #42                     ; Tile 46.
    jmp     .bottom_left
.remove_33
    lda     #0
    jmp     .bottom_left
.remove_35_left
    lda     #34
    jmp     .bottom_left
.remove_44_left
    lda     #40
.bottom_left
    sta     ($fe),y
    iny
    lda     ($fe),y
    cmp     #40
    beq     .remove_40
    cmp     #41
    beq     .remove_41_right
    cmp     #44
    beq     .remove_44_right
    lda     #37                     ; Tile 45.
    jmp     .done
.remove_40
    lda     #0
    jmp     .done
.remove_41_right
    lda     #36
    jmp     .done
.remove_44_right
    lda     #33
.done
    sta     ($fe),y
    ; Fall through!

; Do all processing surrounding a coin pickup.
process_remove_coin subroutine
    lda     next_coin_sound
    eor     #1
    sta     next_coin_sound
    jsr     DOTUNE
    ldx     #score_coin_offset
    jsr     increase_score
    ldy     active_player
    ldx     remaining_coins,y
    dex
    stx     remaining_coins,y
    beq     open_exits
    lda     coin_action_table,x
    beq     .done
    bmi     .show_cash_bag
    ldx     bonus_tile_id
    stx     bonus_tile
    inx
    stx     bonus_tile + 1
    inx
    stx     bonus_tile + 40
    inx
    stx     bonus_tile + 41
    lda     #bonus_display_time
    sta     bonus_timer
    rts
.show_cash_bag
    ldx     #89 | 128
    stx     cash_bag_tile
    inx
    stx     cash_bag_tile + 1
    inx
    stx     cash_bag_tile + 40
    inx
    stx     cash_bag_tile + 41
    lda     #bonus_display_time
    sta     cash_timer
    lda     #7                      ; Start the music.
    jsr     DOTUNE
.done
    rts

; Open the exit doors with the "OUT" sign.
open_exits subroutine
    ldx     #0                      ; Open the escape doors.
    stx     $200d
    stx     $200e
    dec     exits_open              ; Set the flag.  Sign bit indicates open.
    ; Fall through to display the out sign.

; Show the out sign based on the current flag.
show_out_sign subroutine
    ldx     #49
    lda     exits_open
    and     #1
    bne     .top_sign
    stx     $23d9
    inx
    stx     $23da
    sta     $2035                   ; A is 0.
    sta     $2036
    jmp     .flip_flag
.top_sign
    stx     $2035
    inx
    stx     $2036
    lda     #0
    sta     $23d9
    sta     $23da
.flip_flag
    lda     exits_open
    eor     #1
    sta     exits_open
    lda     #exit_timer_length
    sta     exit_timer
    rts

; Handle green gate logic.  Expects the gate block
; address in the y register.
; Check for door obstruction
;   -> If found...
;       -> Reset if center gate
;       -> Skip ahead if moving gate
;       -> Go to spawn state
; If in spawn state...
;   -> Check for player or gate obstructions
;       -> If found...
;           -> Reset if center gate
;           -> Skip ahead if moving gate
;   -> Go to close state
;   -> Set frame timer
; If in close state...
;   -> Decrement timer, if expired...
;       -> Go to next frame
;       -> If last frame...
;           -> Go to hold state
;           -> Set hold timer
;       -> If not last frame...
;           -> Reset close timer
; If in hold state...
;   -> Decrement timer, if expired...
;       -> Go to open state
;       -> Set frame timer
; If in open state...
;   -> Decrement timer, if expired...
;       -> Go to next frame
;       -> If last frame...
;           -> If moving gate...
;               -> Advance index
;               -> Set new address
;               -> Go to spawn state
;       -> If not last frame...
;           -> Reset open timer
process_gate subroutine
    sty     $fd                     ; Save the gate pointer.
    lda     gate_address,y          ; Get the tile address for processing.
    sta     $fe
    lda     gate_address + 1,y
    sta     $ff
    jsr     check_door_obstruction
    bcc     .no_door_obstruction
    jmp     .obstruction
.no_door_obstruction
    ldy     $fd
    lda     gate_phase,y            ; Check the phase.
    bne     .not_spawn
    jmp     .spawn_gate
.not_spawn
    cmp     #gate_close
    bne     .not_close
    jmp     .close_gate
.not_close
    cmp     #gate_hold
    bne     .open_gate
    jmp     .hold_gate
.open_gate
    ldx     gate_timer,y
    cpx     #1
    beq     .open_next
    jmp     .decrement_timer
.open_next
    ldy     #0
    lda     ($fe),y
    cmp     #51
    beq     .finish_open
    cmp     #63
    bne     .continue_open
    lda     #33                     ; Coin version.
    sta     ($fe),y
    lda     #40
    jmp     .finish_open_top
.finish_open
    lda     #0
    sta     ($fe),y
.finish_open_top
    iny
    sta     ($fe),y
    ldy     #40
    lda     ($fe),y
    cmp     #57
    beq     .no_coin_open_bottom
    lda     #36
    sta     ($fe),y
    lda     #34
    jmp     .finish_open_bottom
.no_coin_open_bottom
    lda     #0
    sta     ($fe),y
.finish_open_bottom
    iny
    sta     ($fe),y
    ldy     $fd
    lda     #gate_spawn
    sta     gate_phase,y
    ldx     gate_index,y
    bmi     .no_gate_move           ; Center gate, doesn't move.
    lda     gate_low_byte,x
    sta     gate_address,y
    lda     gate_high_byte,x
    sta     gate_address + 1,y
    cpx     #17
    bne     .no_rollover
    ldx     #-1
.no_rollover
    inx
    stx     gate_index,y
.no_gate_move
    ldx     #gate_frame_time
    jmp     .store_timer
.continue_open
    sec
    sbc     #1
    sta     ($fe),y
    iny
    lda     ($fe),y
    sbc     #1
    sta     ($fe),y
    ldy     #40
    lda     ($fe),y
    sbc     #1
    sta     ($fe),y
    iny
    lda     ($fe),y
    sbc     #1
    sta     ($fe),y
    ldy     $fd
    ldx     #gate_frame_time
    jmp     .store_timer
.obstruction
    ldy     $fd                     ; Get the gate pointer.
    cpy     #moving_gate_1          ; Is this a center gate?
    bcc     .obstructed_center_gate
    ldy     skip_gate               ; Find the first opening.
    ldx     gate_index,y
.skip_loop
    lda     gate_low_byte,x
    sta     $fe
    lda     gate_high_byte,x
    sta     $ff
    jsr     check_door_obstruction
    bcs     .next_skip_index
    jsr     check_player_obstruction
    bcs     .next_skip_index
    ldy     $fd
    jsr     check_gate_obstruction
    bcc     .found_skip
.next_skip_index
    cpx     #17
    bne     .no_skip_check_overflow
    ldx     #-1
.no_skip_check_overflow
    inx
    jmp     .skip_loop
.found_skip
    ldy     $fd                     ; Get the gate back.
    lda     $fe                     ; Update the address.
    sta     gate_address,y
    lda     $ff
    sta     gate_address + 1,y
    cpy     skip_gate               ; Is this the skip gate?
    bne     .flip_skip_gate
    cpx     #17
    bne     .no_flip_skip_overflow
    ldx     #-1
.no_flip_skip_overflow
    inx 
    stx     gate_index,y            ; Update the index.
.flip_skip_gate
    ldy     #moving_gate_1
    cpy     skip_gate
    bne     .skip_to_gate_1
    ldy     #moving_gate_2
.skip_to_gate_1
    sty     skip_gate
.obstructed_center_gate
    ldy     $fd                     ; Get the gate back.
    lda     #1
    sta     gate_timer,y
    lda     #gate_spawn
    sta     gate_phase,y
    rts
.spawn_gate
    ldx     gate_timer,y
    cpx     #1
    beq     .spawn_timer_expired
    jmp     .decrement_timer
.spawn_timer_expired
    jsr     check_player_obstruction
    bcs     .obstruction
    ldy     $fd
    jsr     check_gate_obstruction
    bcs     .obstruction
    lda     #gate_close
    sta     gate_phase,y
    sta     gate_timer,y            ; gate_close is 1.
.close_gate
    ldx     gate_timer,y
    cpx     #1
    bne     .decrement_timer
    ldy     #0
    lda     ($fe),y
    beq     .new_close
    cmp     #33
    bne     .continue_close
    lda     #63                     ; Use the coin tile.
    sta     ($fe),y
    lda     #66
    jmp     .finish_top
.new_close
    lda     #51
    sta     ($fe),y
    lda     #54
.finish_top
    iny
    sta     ($fe),y
    ldy     #40
    lda     ($fe),y
    beq     .no_coin_bottom
    lda     #69
    sta     ($fe),y
    lda     #72
    jmp     .finish_bottom
.no_coin_bottom
    lda     #57
    sta     ($fe),y
    lda     #60
.finish_bottom
    iny
    sta     ($fe),y
    jmp     .finish_close
.continue_close
    clc
    adc     #1
    sta     ($fe),y
    iny
    lda     ($fe),y
    adc     #1
    sta     ($fe),y
    ldy     #40
    lda     ($fe),y
    adc     #1
    sta     ($fe),y
    iny
    lda     ($fe),y
    adc     #1
    sta     ($fe),y
.finish_close
    ldy     $fd                     ; Retrieve the block pointer.
    cmp     #62                     ; Check for closure.
    beq     .go_to_hold
    cmp     #74                     ; Coin version.
    beq     .go_to_hold
    ldx     #gate_frame_time
    jmp     .store_timer
.go_to_hold
    lda     #gate_hold
    sta     gate_phase,y
    ldx     #gate_hold_time
    jmp     .store_timer
.hold_gate
    ldx     gate_timer,y
    cpx     #1
    bne     .decrement_timer
    lda     #gate_open
    sta     gate_phase,y
    lda     #gate_frame_time
    sta     gate_timer,y
    rts
.decrement_timer
    dex
.store_timer
    stx     gate_timer,y
    rts

; Check to see if a player door is obstructing a
; gate location.  Expects the location in $fe-$ff.
; Sets the carry bit if obstructed, clears it if not.
; Leaves the last tile checked in the accumulator.
check_door_obstruction subroutine
    ldy     #0
    lda     ($fe),y                 ; See if there's a door in the way.
    bmi     .obstruction
    iny
    lda     ($fe),y
    bmi     .obstruction
    clc                             ; No obstruction.
    rts
.obstruction
    sec
    rts

; Check to see if a player is obstructing a gate.
; Sets the carry if so, clears it if not.  Expects
; the tile to check in $fe-$ff.
check_player_obstruction subroutine
    ldy     #player_1_data
    jsr     .run_check
    bcs     .done
    ldy     #cop_1_data
    jsr     .run_check
    bcs     .done
    ldy     #cop_2_data
    jsr     .run_check
    bcs     .done
    ldy     #cop_3_data
    jsr     .run_check
    bcs     .done
    ldy     #cop_4_data
    jmp     .run_check
.done
    rts
.run_check
    txa
    pha
    ldx     player_tile_id,y
    lda     tile_map_low_byte,x
    sta     $f0
    lda     tile_map_high_byte,x
    sta     $f1
    pla
    tax
    lda     player_direction,y
    bne     .player_moving
.no_obstruction
    clc
    rts                             ; Not moving, not obstruction.
.player_moving
    and     [#direction_left | #direction_right]
    bne     .horizontal_move
    lda     $f0
    cmp     $fe
    bne     .not_on_tile_v
    lda     $f1
    cmp     $ff
    bne     .not_on_tile_v
    lda     player_tile_y,y
    cmp     #9
    bcc     .obstruction
    clc
    rts
.not_on_tile_v
    lda     $fe                     ; See if the player is up one tile.
    sec
    sbc     #80
    sta     $fa
    lda     $ff
    sbc     #0
    sta     $fb
    lda     $f0
    cmp     $fa
    bne     .no_obstruction
    lda     $f1
    cmp     $fb
    bne     .no_obstruction
    lda     player_tile_y,y
    cmp     #7
    bcc     .no_obstruction
    rts
.horizontal_move
    lda     $f0
    cmp     $fe
    bne     .not_on_tile_h
    lda     $f1
    cmp     $ff
    bne     .not_on_tile_h
    lda     player_tile_x,y
    cmp     #5
    bcc     .obstruction
    clc
    rts                             ; No obstructiom, carry is clear.
.not_on_tile_h
    lda     $fe                     ; See if the player is on the tile
    sec                             ; to the left.
    sbc     #2
    sta     $fa
    lda     $ff
    sbc     #0
    sta     $fb
    lda     $f0
    cmp     $fa
    bne     .no_obstruction
    lda     $f1
    cmp     $fb
    bne     .no_obstruction
    lda     player_tile_x,y
    cmp     #4
    bcc     .no_obstruction
    rts                             ; Obstruction, carry is set.
.obstruction
    sec                             ; Return value.
    rts

; See if a gate location is blocked by another gate.
; Expects the gate location in $fe-$ff and the gate
; being checked in y.  Sets the carry bit if the
; location is obstructed.
check_gate_obstruction subroutine
    cpy     #moving_gate_1
    beq     .check_gate_2
    bcc     .no_obstruction         ; Center gate, ignore.
    lda     moving_gate_1 + gate_address
    cmp     $fe
    bne     .no_obstruction
    lda     moving_gate_1 + gate_address + 1
    cmp     $ff
    bne     .no_obstruction
.obstruction
    sec
    rts
.check_gate_2
    lda     moving_gate_2 + gate_address
    cmp     $fe
    bne     .no_obstruction
    lda     moving_gate_2 + gate_address + 1
    cmp     $ff
    beq     .obstruction
.no_obstruction
    clc
    rts

; Add points to the score.  Expects the offset into
; the score values table in x.
increase_score subroutine
    sed
    clc
    lda     active_score
    adc     score_table,x
    sta     active_score
    lda     active_score + 1
    adc     score_table + 1,x
    sta     active_score + 1
    lda     active_score + 2
    adc     score_table + 2,x
    sta     active_score + 2
    cld
    cmp     #16                     ; Check for overflow.
    bcc     .check_extra_life       ; No overflow, just display.
    lda     #$99                    ; Max out the score.
    sta     active_score
    sta     active_score + 1
    lda     #$09
    sta     active_score + 2
.check_extra_life
    ldy     active_player
    lda     player_1_extra,y        ; Check for extra life at 15,000.
    bne     display_score           ; Already awarded.
    lda     active_score + 1
    cmp     #$15
    bmi     display_score
    ldx     player_1_lives,y
    inx
    txa
    sta     player_1_lives,y
    inc     lives_display
    sta     player_1_extra,y        ; A is not 0.
    lda     #$0f                    ; Play the sound.
    jsr     DOTUNE
    ; Fall through into score rendering.

; Display the score on the screen.
display_score subroutine
    ldy     #2                      ; Highest byte of score.
    ldx     #0                      ; Position to write to.
    lda     active_score + 2        ; Find the first non-0 digit.
    bne     .found_start_byte
    dey                             ; Check the middle byte.
    lda     active_score + 1
    bne     .found_start_byte
    dey
    lda     active_score            ; Only one byte left.
.found_start_byte
    pha                             ; Save the byte.
    cmp     #16                     ; Ignore the high nybble if 0.
    bcs     .draw_high_nybble
    jmp     .draw_low_nybble
.remaining_score_loop               ; Loop through the remaining bytes.
    dey
    bmi     .done
    lda     active_score,y          ; Get the next byte.
    pha                             ; Save it.
.draw_high_nybble
    lsr
    lsr
    lsr
    lsr                             ; Shift it over.
    clc
    adc     #digit_base
    sta     score_base,x
    inx                             ; Next score position.
.draw_low_nybble
    pla                             ; Retrive the byte.
    and     #$0f                    ; Mask off the high nybble.
    clc
    adc     #digit_base
    sta     score_base,x
    inx
    jmp     .remaining_score_loop
.done
    lda     #digit_base             ; End with a 0.
    sta     score_base,x
    rts

; Turn on the score bonus display.  Place the offset
; into the bonus table in x.
display_bonus subroutine
    lda     #0                      ; Blank the fifth place just
    sta     bonus_base + 4          ; in case there's already a score.
    lda     #88                     ; Place the +.
    sta     bonus_base
    lda     bonus_table,x
    sta     bonus_base + 1
    ldy     bonus_table + 1,x
    lda     #78
.zero_loop
    sta     bonus_base + 1,y
    dey
    bne     .zero_loop
    lda     #bonus_timer_length     ; Start the bonus timer.
    sta     game_timer
    rts

; Clear the bonus display text.
hide_bonus_display subroutine
    ldy     #4
    lda     #0
.loop
    sta     bonus_base,y
    dey
    bpl     .loop
    rts

; Hide the cash bag.
hide_cash_bag subroutine
    lda     #0
    sta     cash_bag_tile
    sta     cash_bag_tile + 1
    sta     cash_bag_tile + 40
    sta     cash_bag_tile + 41
    lda     #7                      ; Stop the music.
    jsr     KILLTUNE
    rts

; Hide the bonus object.
hide_bonus subroutine
    lda     #0
    sta     bonus_tile
    sta     bonus_tile + 1
    sta     bonus_tile + 40
    sta     bonus_tile + 41
    rts

; Copy the page in memory addressed in $fc-$fd
; to the page addressed in $fe-$ff.
copy_page subroutine
    ldy     #0
.loop
    lda     ($fc),y
    sta     ($fe),y
    dey
    bne     .loop
    rts

; Erase the green gate whose address is in y.
erase_gate subroutine
    lda     gate_phase,y
    beq     .clear_struct
    sty     $fd
    lda     gate_address,y
    sta     $fe
    lda     gate_address + 1,y
    sta     $ff
    ldy     #0
    lda     ($fe),y
    cmp     #51
    bmi     .clear_struct
    cmp     #63
    bmi     .no_top_coin
    lda     #33
    sta     ($fe),y
    lda     #40
    jmp     .finish_top
.no_top_coin
    lda     #0
    sta     ($fe),y
.finish_top
    iny
    sta     ($fe),y
.check_bottom
    ldy     #40
    lda     ($fe),y
    cmp     #63
    bmi     .no_bottom_coin
    lda     #36
    sta     ($fe),y
    lda     #34
    jmp     .finish_bottom
.no_bottom_coin
    lda     #0
    sta     ($fe),y
.finish_bottom
    iny
    sta     ($fe),y
.clear_struct
    ldy     $fd
    lda     #0
    sta     gate_address,y
    sta     gate_address + 1,y
    sta     gate_timer,y
    sta     gate_index,y
    rts

; Erase the door whose address is in y.
erase_door subroutine
    ldx     active_player
    lda     remaining_coins,x
    cmp     #total_coins
    beq     .clear_struct           ; New stage, already overwritten.
    lda     door_address + 1,y
    beq     .no_door
    sty     $fd
    lda     door_address,y
    sta     $fe
    lda     door_address + 1,y
    sta     $ff
    lda     #0
    tay
    sta     ($fe),y
    iny
    sta     ($fe),y
    ldy     #40
    sta     ($fe),y
    iny
    sta     ($fe),y
    ldy     $fd
.clear_struct
    lda     #0                      ; Needed if reached from branch.
    sta     door_address,y
    sta     door_address + 1,y
    sta     door_flags,y
    sta     door_timer,y
.no_door
    rts

; Prepare the current player's color pattern.
copy_color_pattern subroutine
    lda     active_player
    beq     .rob_colors
    mwa     banks_color_pattern,$fe
    jmp     .copy_colors
.rob_colors
    mwa     rob_color_pattern,$fe
.copy_colors
    ldy     #15
.loop
    lda     ($fe),y
    sta     player_color_pattern,y
    dey
    bpl     .loop
    rts

; Copy the current score into long-term storage.
backup_score subroutine
    lda     active_player
    beq     .copy_player_1
    lda     #<player_2_score
    jmp     .copy_score
.copy_player_1
    lda     #<player_1_score
.copy_score
    sta     $fe
    ldy     #3
    sty     $ff
    dey
.loop
    lda     active_score,y
    sta     ($fe),y
    dey
    bpl     .loop
    rts

; Vertical blank interrupt for the game.
game_vblank subroutine
    lda     $fe
    pha
    lda     $ff
    pha
    jsr     TUNER
    pla
    sta     $ff
    pla
    sta     $fe
    lda     #0
    sta     ATRACT
    lda     keypad_cooldown
    beq     .no_keypad_cooldown
    dec     keypad_cooldown
.no_keypad_cooldown
    jmp     ENDINT
