; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; First intermission, the one with the car

frame_direction equ $3ef
frame_offset    equ $3ee
player_xpos     equ $3ed
cop_xpos        equ $3ec
player_subpos   equ $3eb
cop_subpos      equ $3ea
player_timer    equ $3e9

; Intermission entry point.  Reuse the player intro display list.
start_intermission_1 subroutine
    jsr     start_transition
    mwa     player_intro_display_list,sDLISTL
    mwa     ready_screen_interrupt,VDSLST
    mwa     intermission_vblank,VVBLKD
    jsr     build_car_display_list
    lda     active_player
    asl
    asl
    asl
    asl
    sta     rider_color_offset
    tax
    lda     rob_color_pattern,x
    sta     sCOLPM0
    jsr     reset_cop_colors
    lda     #0
    sta     sCOLOR1
    sta     sCOLOR2
    sta     HPOSP0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM3
    sta     HPOSM2
    sta     HPOSM1
    sta     HPOSM0
    sta     player_subpos
    sta     cop_subpos
    sta     player_xpos
    sta     cop_xpos
    sta     frame_direction
    sta     car_scroll
    sta     HSCROL
    sta     car_frame
    tay
.clear_car_zone_loop
    sta     $3400,y
    sta     $3500,y
    sta     $3600,y
    sta     $3700,y
    sta     $3800,y
    sta     $3900,y
    dey
    bne     .clear_car_zone_loop
    jsr     clear_sprites
    lda     #$14
    sta     PRIOR
    lda     #car_frame_interval
    sta     animation_timer
    lda     #28
    sta     rider_pos
    lda     #$36
    sta     sCOLOR0
    lda     #5
    sta     player_timer
    lda     #15
    sta     frame_offset
    jsr     complete_transition
    lda     #$50
    sta     AUDCTL
    lda     #$16
    jsr     DOTUNE
    lda     #$17
    jsr     DOTUNE
    lda     #$18
    jsr     DOTUNE
    lda     #$19
    jsr     DOTUNE

intermission_1_phase_1_loop subroutine
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_1
.no_cancel
    dec     player_timer
    bne     .no_animation_update
    lda     #5
    sta     player_timer
    jsr     intermission_1_update_players
.no_animation_update
    ldy     player_xpos
    cpy     #1
    beq     .no_player_move
    lda     player_subpos
    clc
    adc     #45
    cmp     #60
    bmi     .update_player_subpos
    dey
    sty     player_xpos
    sty     HPOSP0
    sec
    sbc     #60
.update_player_subpos
    sta     player_subpos
.no_player_move
    cpy     #180
    bcs     intermission_1_phase_1_loop
    ldy     cop_xpos
    lda     cop_subpos
    clc
    adc     #55
    cmp     #60
    bmi     .update_cop_subpos
    dey
    sty     cop_xpos
    sty     HPOSP1
    sec
    sbc     #60
.update_cop_subpos
    sta     cop_subpos
    lda     cop_xpos
    jsr     apply_cop_positions
    ldy     player_xpos
    cpy     #1
    bne     intermission_1_phase_1_loop
    cmp     #1
    bne     intermission_1_phase_1_loop

intermission_1_phase_2 subroutine
    inc     frame_direction
    lda     #28
    sta     HPOSP0
    lda     #0
    ldy     #15
.erase_player_loop
    sta     $1475,y
    dey
    bpl     .erase_player_loop
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_1
.no_cancel
    dec     player_timer
    bne     .no_animation_update
    lda     #5
    sta     player_timer
    jsr     intermission_1_update_players
.no_animation_update
    lda     cop_xpos
    cmp     #215
    beq     .no_cop_move
    clc
    adc     #1
    sta     cop_xpos
    jsr     apply_cop_positions
    cmp     #100
    bcc     .loop
.no_cop_move
    lda     rider_pos
    cmp     #$e0
    bne     .not_done
end_intermission_1
    jsr     check_preview_mode      ; Back to title if this is a preview.
    mwa     game_display_list,sDLISTL
    mwa     game_dli,VDSLST
    mwa     game_vblank,VVBLKD
    lda     #$11
    sta     PRIOR
    lda     #$86
    sta     sCOLOR0
    lda     #$1e
    sta     sCOLOR1
    lda     #gate_color
    sta     sCOLOR2
    jsr     clear_sprites
    jmp     start_new_stage
.not_done
    jsr     process_car
    jmp     .loop

; Check to see if this intermission was accessed from the preview.
check_preview_mode subroutine
    lda     intermission
    bne     .back_to_title
    rts
.back_to_title
    ldx     #$ff
    txs
    jmp     start_title_screen

; Wait for vblank and then check to see if the
; intermission has been cancelled.  Sets 0 bit if
; intermission should end.
check_intermission_cancel subroutine
    jsr     wait_for_vb
    ldy     active_stick
    lda     TRIG0,y
    rts

; Apply cop horizontal positions.  Expects the position
; in the accumulator.
apply_cop_positions subroutine
    sta     HPOSP1
    clc
    adc     #10
    sta     HPOSP2
    adc     #10
    sta     HPOSP3
    adc     #10
    sta     HPOSM3
    adc     #2
    sta     HPOSM2
    adc     #2
    sta     HPOSM1
    adc     #2
    sta     HPOSM0
    rts

; Update player images.
intermission_1_update_players subroutine
    ldx     #15
    ldy     frame_offset
    cpy     #15
    beq     .switch_to_31
    lda     #15
    jmp     .frame_ready
.switch_to_31
    lda     #31
.frame_ready
    sta     frame_offset
    lda     frame_direction
    bne     .frame_right_loop
.frame_left_loop
    lda     player_left_1,y
    sta     $1475,x
    lda     cop_left_1,y
    sta     $1375,x
    sta     $1575,x
    sta     $1675,x
    sta     $1775,x
    dey
    dex
    bpl     .frame_left_loop
    rts
.frame_right_loop
    lda     cop_right_1,y
    sta     $1375,x
    sta     $1575,x
    sta     $1675,x
    sta     $1775,x
    dey
    dex
    bpl     .frame_right_loop
    rts

; Common vertical blank routine for intermissions.
intermission_vblank subroutine
    lda     $ff
    pha
    lda     $fe
    pha
    jsr     TUNER
    pla
    sta     $fe
    pla
    sta     $ff
    lda     #0
    sta     ATRACT
    jmp     ENDINT

