; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Logic for the second intermission, with the phones

cop_animation_timer equ $3ff
cop_frame_offset    equ $3fe
cop_pos             equ $3fd
phone_player_pos    equ $3fc
cash_player_pos     equ $3fb
phone_jitter        equ $3fa

cop_frame           equ $f0
phone_player_frame  equ $f2
cash_player_frame   equ $f4

; Prepare the intermission.
start_intermission_2 subroutine
    jsr     start_transition
    mwa     intermission_2_display_list,sDLISTL
    mwa     intermission_2_interrupt,VDSLST
    mwa     intermission_vblank,VVBLKD
    jsr     clear_sprites
    lda     active_player
    beq     .rob_is_cash_player
    mwa     banks_color_pattern,$fc
    jmp     .write_cash_pattern
.rob_is_cash_player
    mwa     rob_color_pattern,$fc
.write_cash_pattern
    mwa     $3e00,$fe
    ldy     #16
    jsr     copy_memory
    jsr     reset_cop_colors
    lda     #$34
    sta     $ff
    jsr     clear_page
    ldy     #31
.phone_loop
    lda     bonus_phone,y
    sta     [$3000 + tileset_length],y
    dey
    bpl     .phone_loop
    ldy     #93 | #128
    sty     $3443
    iny
    sty     $3444
    iny
    sty     $346b
    iny
    sty     $346c
    ldy     #89 | #128
    sty     $34bd
    iny
    sty     $34be
    iny
    sty     $34e5
    iny
    sty     $34e6
    mwa     cop_left_1,cop_frame
    mwa     player_up_1,phone_player_frame
    mwa     player_left_1,cash_player_frame
    lda     #0
    sta     cop_frame_offset
    sta     phone_jitter
    sta     VSCROL
    ldx     #$80
    stx     cop_pos
    jsr     intermission_2_draw_cop
    lda     #20
    sta     cop_animation_timer
    lda     #120
    sta     game_timer
    lda     #156
    sta     HPOSP1
    lda     #92
    sta     HPOSP0
    lda     #200
    sta     HPOSP2
    sta     cash_player_pos
    lda     #$e0
    sta     phone_player_pos
    jsr     complete_transition
    lda     #$50
    sta     AUDCTL
    lda     #$12
    jsr     DOTUNE
    lda     #$13
    jsr     DOTUNE
    lda     #$14
    jsr     DOTUNE
    lda     #$15
    jsr     DOTUNE
    ; Fall into the first loop.

; Phase 1: cop stands briefly in place.
intermission_2_phase_1_loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    dec     game_timer
    beq     intermission_2_phase_2_loop
    dec     cop_animation_timer
    bne     intermission_2_phase_1_loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jmp     intermission_2_phase_1_loop

; Phase 2: non-active player walks up to phone.
intermission_2_phase_2_loop subroutine
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    ldy     phone_player_pos
    cpy     #$96
    beq     intermission_2_phase_3_loop
    dec     phone_player_pos
    jsr     intermission_2_draw_phone_player
    dec     cop_animation_timer
    bne     intermission_2_phase_2_loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jmp     intermission_2_phase_2_loop

; Phase 3: phone rings.
intermission_2_phase_3_loop subroutine
    lda     #30
    sta     game_timer
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    dec     game_timer
    beq     intermission_2_phase_4_loop
    lda     game_timer
    and     #2
    sta     VSCROL
.no_jitter
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jsr     intermission_2_draw_phone_player
    jmp     .loop

; Phase 4: cop walks to phone.
intermission_2_phase_4_loop subroutine
    mwa     cop_up_1,cop_frame
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    ldy     cop_pos
    cpy     #$4d
    beq     intermission_2_phase_5_loop
    dec     cop_pos
    jsr     intermission_2_draw_cop
    lda     cop_animation_timer
    and     #2
    eor     #2                      ; This is off by one frame.
    sta     VSCROL
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_phone_player
    jmp     .loop

; Phase 5: brief delay
intermission_2_phase_5_loop subroutine
    lda     #30
    sta     game_timer
    lda     #0
    sta     VSCROL
.loop
    dec     game_timer
    beq     intermission_2_phase_6_loop
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jsr     intermission_2_draw_phone_player
    jmp     .loop

; Phase 6: active player walks to cash bag.
intermission_2_phase_6_loop subroutine
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    ldy     cash_player_pos
    cpy     #164
    beq     intermission_2_phase_7_loop
    dey
    sty     cash_player_pos
    sty     HPOSP2
    jsr     intermission_2_draw_cash_player
    dec     cop_animation_timer
    bne     intermission_2_phase_6_loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jsr     intermission_2_draw_phone_player
    jmp     intermission_2_phase_6_loop

; Phase 7: active player walks off screen.
intermission_2_phase_7_loop subroutine
    lda     #0                      ; Remove the cash bag.
    sta     $34bd
    sta     $34be
    sta     $34e5
    sta     $34e6
    mwa     player_right_1,cash_player_frame
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    ldy     cash_player_pos
    cpy     #220
    beq     intermission_2_phase_8_loop
    iny
    sty     cash_player_pos
    sty     HPOSP2
    jsr     intermission_2_draw_cash_player
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jsr     intermission_2_draw_phone_player
    jmp     .loop

; Phase 8: non-active player walks down, cop returns to bag.
intermission_2_phase_8_loop subroutine
    mwa     player_down_1,phone_player_frame
    mwa     cop_down_1,cop_frame
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    inc     phone_player_pos
    jsr     intermission_2_draw_phone_player
    ldy     cop_pos
    cpy     #$80
    beq     intermission_2_phase_9_loop
    inc     cop_pos
    jsr     intermission_2_draw_cop
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jmp     .loop

; Phase 9: cop gets angry.
intermission_2_phase_9_loop subroutine
    mwa     cop_angry_1,cop_frame
    lda     #door_color
    sta     sCOLPM1
    lda     #120
    sta     game_timer
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_2
.no_cancel
    dec     game_timer
    beq     end_intermission_2
    lda     phone_player_pos
    cmp     #$e0
    beq     .hide_phone_player
    inc     phone_player_pos
    jmp     .phone_player_done
.hide_phone_player
    lda     #0
    sta     HPOSP0
.phone_player_done
    jsr     intermission_2_draw_phone_player
    dec     cop_animation_timer
    bne     .loop
    jsr     intermission_2_process_animation
    jsr     intermission_2_draw_cop
    jmp     .loop
end_intermission_2
    jsr     check_preview_mode
    mwa     game_display_list,sDLISTL
    mwa     game_dli,VDSLST
    mwa     game_vblank,VVBLKD
    jsr     clear_sprites
    jmp     start_new_stage

; Handle the animation sequence.
intermission_2_process_animation subroutine
    lda     #20
    sta     cop_animation_timer
    lda     cop_frame_offset
    eor     #16
    sta     cop_frame_offset
    rts

; Draw the cop frame at the current position.
intermission_2_draw_cop subroutine
    ldy     cop_frame_offset
    ldx     cop_pos
    lda     #0
    sta     $14ff,x
.loop
    lda     (cop_frame),y
    sta     $1500,x
    inx
    iny
    cpy     #16
    beq     .done
    cpy     #32
    bne     .loop
.done
    lda     #0
    sta     $1500,x
    rts

; Draw the inactive player sprite.
intermission_2_draw_phone_player subroutine
    lda     active_player
    bne     .rob_color
    mwa     banks_color_pattern,$fe
    jmp     .color_ready
.rob_color
    mwa     rob_color_pattern,$fe
.color_ready
    ldy     cop_frame_offset
    ldx     phone_player_pos
    lda     #0
    sta     $13ff,x
.loop
    lda     (phone_player_frame),y
    sta     $1400,x
    inx
    iny
    cpy     #16
    beq     .done_player
    cpy     #32
    bne     .loop
.done_player
    lda     #0
    tay
    sta     $1400,x
    ldy     #0
    ldx     phone_player_pos
.color_loop
    lda     ($fe),y
    sta     $3e00,x
    inx
    iny
    cpy     #16
    bne     .color_loop
    rts

; Draw the active player sprite.
intermission_2_draw_cash_player subroutine
    ldy     cop_frame_offset
    ldx     #0
    txa
.loop
    lda     (cash_player_frame),y
    sta     $1680,x
    inx
    iny
    cpy     #16
    beq     .done
    cpy     #32
    bne     .loop
.done
    rts

