; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Code for the hat intermission

; Constants.
int_3_animation_length  equ #10
int_3_collapse_length   equ #6

; Intermission data.
player_hpos             equ $3ff
player_vpos             equ $3fe
animation_offset        equ $3fd
int_3_animation_timer   equ $3fc
collapse_row_count      equ $3fb
cop_hpos                equ $3fa
cop_turns_remaining     equ $3f9

player_frame    equ $f0
int_3_cop_frame       equ $f2

; Prepare the intermission.
start_intermission_3 subroutine
    jsr     start_transition
    mwa     intermission_3_display_list,sDLISTL
    mwa     intermission_vblank,VVBLKD
    jsr     clear_sprites
    ldy     #31
.hat_loop
    lda     bonus_hat,y
    sta     [$3000 + tileset_length],y
    dey
    bpl     .hat_loop
    lda     #int_3_animation_length
    sta     int_3_animation_timer
    lda     #$80
    sta     player_vpos
    lda     #8
    sta     cop_turns_remaining
    mwa     player_right_1,player_frame
    mwa     cop_right_1,int_3_cop_frame
    jsr     reset_cop_colors
    lda     #0
    sta     player_hpos
    sta     cop_hpos
    sta     HPOSP0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM0
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    sta     animation_offset
    ldy     #79
.clear_loop
    sta     $3400,y
    dey
    bpl     .clear_loop
    ldy     #93
    sty     $340e
    sty     $3410
    sty     $3412
    sty     $3416
    sty     $3418
    iny
    sty     $340f
    sty     $3411
    sty     $3413
    sty     $3417
    sty     $3419
    iny
    sty     $3436
    sty     $3438
    sty     $343a
    sty     $343e
    sty     $3440
    iny
    sty     $3437
    sty     $3439
    sty     $343b
    sty     $343f
    sty     $3441
    jsr     complete_transition
    lda     #$50
    sta     AUDCTL
    lda     #$1a
    jsr     DOTUNE
    lda     #$1b
    jsr     DOTUNE
    lda     #$1c
    jsr     DOTUNE
    lda     #$1d
    jsr     DOTUNE

; Phase 1: player runs on from left.
intermission_3_phase_1_loop subroutine
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    ldx     player_hpos
    cpx     #128
    beq     intermission_3_phase_2_loop
    inx
    stx     player_hpos
    stx     HPOSP0
    dec     int_3_animation_timer
    bne     intermission_3_phase_1_loop
    jsr     intermission_3_process_animation
    jsr     intermission_3_draw_player
    jmp     intermission_3_phase_1_loop

; Phase 2: player runs up into hat row.
intermission_3_phase_2_loop subroutine
    mwa     player_up_1,player_frame
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    ldx     player_vpos
    cpx     #$5d
    beq     intermission_3_phase_3_loop
    dec     player_vpos
    jsr     intermission_3_draw_player
    dec     int_3_animation_timer
    bne     .loop
    jsr     intermission_3_process_animation
    jmp     .loop

; Phase 3: player hides in hat.
intermission_3_phase_3_loop subroutine
    lda     #1
    sta     int_3_animation_timer
    lda     #6
    sta     collapse_row_count
    lda     active_player
    beq     .rob
    mwa     banks_happy_pattern_1,$fe
    jmp     .color_ready
.rob
    mwa     rob_happy_pattern_1,$fe
.color_ready
    ldy     #15
.setup_loop
    lda     player_happy_1,y
    sta     $145d,y
    lda     ($fe),y
    sta     $3e5e,y
    dey
    bpl     .setup_loop
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    dec     int_3_animation_timer
    bne     .loop
    lda     #int_3_collapse_length
    sta     int_3_animation_timer
    dec     collapse_row_count
    beq     intermission_3_phase_4_loop
    lda     #$6c
    sec
    sbc     collapse_row_count
    tax
    ldy     #9
.crumple_loop
    lda     $1400,x
    sta     $1401,x
    lda     $3e01,x
    sta     $3e02,x
    dex
    dey
    bne     .crumple_loop
    jmp     .loop

; Phase 4: cops run to center from left and stop.
intermission_3_phase_4_loop subroutine
    lda     #1
    sta     int_3_animation_timer
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    lda     cop_hpos
    cmp     #138
    beq     intermission_3_phase_5_loop
    clc
    adc     #1
    sta     cop_hpos
    sta     HPOSP1
    sec
    sbc     #10
    sta     HPOSP2
    sbc     #10
    sta     HPOSP3
    sbc     #4
    sta     HPOSM0
    sbc     #2
    sta     HPOSM1
    sbc     #2
    sta     HPOSM2
    sbc     #2
    sta     HPOSM3
    dec     int_3_animation_timer
    bne     .loop
    jsr     intermission_3_process_animation
    jsr     intermission_3_draw_cops
    jmp     .loop

; Phase 5: cops look around.
intermission_3_phase_5_loop subroutine
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    dec     int_3_animation_timer
    bne     intermission_3_phase_5_loop
    jsr     intermission_3_process_animation
    dec     cop_turns_remaining
    bmi     intermission_3_phase_6_loop
    lda     cop_turns_remaining
    and     #1
    beq     .face_left
    mwa     cop_right_1,int_3_cop_frame
    jmp     .frame_ready
.face_left
    mwa     cop_left_1,int_3_cop_frame
.frame_ready
    jsr     intermission_3_draw_cops
    jmp     intermission_3_phase_5_loop

; Phase 6: cops exit to right
intermission_3_phase_6_loop subroutine
    lda     #1
    sta     int_3_animation_timer
    mwa     cop_right_1,int_3_cop_frame
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    lda     cop_hpos
    beq     intermission_3_phase_7_loop
    clc
    adc     #1
    sta     cop_hpos
    sta     HPOSP1
    sec
    sbc     #10
    sta     HPOSP2
    sbc     #10
    sta     HPOSP3
    sbc     #4
    sta     HPOSM0
    sbc     #2
    sta     HPOSM1
    sbc     #2
    sta     HPOSM2
    sbc     #2
    sta     HPOSM3
    dec     int_3_animation_timer
    bne     .loop
    jsr     intermission_3_process_animation
    jsr     intermission_3_draw_cops
    jmp     .loop

; Phase 7: player pops out of hat
intermission_3_phase_7_loop subroutine
    lda     #1
    sta     int_3_animation_timer
    lda     #0
    sta     collapse_row_count
.loop
    jsr     check_intermission_cancel
    bne     .no_cancel
    jmp     end_intermission_3
.no_cancel
    dec     int_3_animation_timer
    bne     .loop
    lda     collapse_row_count
    cmp     #5
    beq     intermission_3_phase_8_loop
    inc     collapse_row_count
    lda     #int_3_collapse_length
    sta     int_3_animation_timer
    lda     active_player
    beq     .rob
    mwa     [banks_happy_pattern_1 + 3],$fe
    jmp     .color_ready
.rob
    mwa     [rob_happy_pattern_1 + 3],$fe
.color_ready
    lda     #$5d
    clc
    adc     #8
    sec
    sbc     collapse_row_count
    tax                             ; Draw target
    lda     #7
    clc
    adc     collapse_row_count
    sta     $fd
    ldy     #0
.draw_loop
    lda     player_happy_1 + 3,y
    sta     $1400,x
    lda     ($fe),y
    sta     $3e01,x
    inx
    iny
    dec     $fd
    bpl     .draw_loop
    jmp     .loop

; Phase 8: player exits to bottom
intermission_3_phase_8_loop subroutine
    mwa     player_down_1,player_frame
    lda     #1
    sta     int_3_animation_timer
.loop
    jsr     check_intermission_cancel
    beq     end_intermission_3
.no_cancel
    lda     player_vpos
    cmp     #$d0
    beq     .final_delay
    inc     player_vpos
    jsr     intermission_3_draw_player
    dec     int_3_animation_timer
    bne     .loop
    jsr     intermission_3_process_animation
    jmp     .loop
.final_delay
    lda     #0
    sta     HPOSP0
    ldx     #60
.final_delay_loop
    jsr     check_intermission_cancel
    beq     end_intermission_3
    dex
    bpl     .final_delay_loop
end_intermission_3
    jsr     check_preview_mode
    mwa     game_display_list,sDLISTL
    mwa     game_vblank,VVBLKD
    jsr     clear_sprites
    jmp     start_new_stage

; Update the animation offset.
intermission_3_process_animation subroutine
    lda     #int_3_animation_length
    sta     int_3_animation_timer
    lda     animation_offset
    eor     #16
    sta     animation_offset
    rts

; Draw the player at the current location.
intermission_3_draw_player subroutine
    lda     active_player           ; Prepare the color pattern.
    beq     .rob_pattern
    mwa     banks_color_pattern,$fe
    jmp     .color_ready
.rob_pattern
    mwa     rob_color_pattern,$fe
.color_ready
    lda     #16
    sta     $fd
    ldx     player_vpos
    lda     #0
    sta     $13ff,x                 ; Erase a byte in front.
    ldy     animation_offset
.player_loop
    lda     (player_frame),y
    sta     $1400,x
    iny
    inx
    dec     $fd
    bne     .player_loop
    lda     #0                      ; Write one more to clean uo vertical.
    sta     $1400,x
    ldx     player_vpos
    ldy     #0
.color_loop
    lda     ($fe),y                 ; This is off by one scanline for some
    sta     $3e01,x                 ; reason, just work around it.
    inx
    iny
    cpy     #16
    bne     .color_loop
    rts

; Draw all four cops using thr current frame.
intermission_3_draw_cops subroutine
    ldy     animation_offset
    ldx     #0
.loop
    lda     (int_3_cop_frame),y
    sta     $1380,x
    sta     $1580,x
    sta     $1680,x
    sta     $1780,x
    iny
    inx
    cpx     #16
    bne     .loop
    rts

