; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Various interrupt routines

    org     $9000

; Main game screen DLI
game_dli subroutine
    pha
    tya
    pha
    txa
    pha

    ldy     #player_min_y
.loop
    ldx     $3e00,y
    stx     COLPM0
    sta     WSYNC
    cpy     #player_bottom_y + 16
    beq     .done
    iny
    jmp     .loop
.done

    pla
    tax
    pla
    tay
    pla
    rti

; Scanline delay macro.
    mac     delay_scanline
    ldy     #{0}
.loop
    sta     WSYNC
    dey
    bne     .loop
    endm

; Title screen interrupts.
title_logo_interrupt
    pha
    tya
    pha

    lda     title_color_1
    sta     COLPF1
    lda     title_color_2
    sta     WSYNC
    sta     COLPF2
    lda     next_title_interrupt
    sta     VDSLST
    lda     next_title_interrupt + 1
    sta     VDSLST + 1
    delay_scanline 40
    sty     COLPF2
    lda     #$e
    sta     COLPF1

    pla
    tay
    pla
    rti

title_interrupt_1
    pha
    tya
    pha

    lda     #$98                    ; Start coloring Rob.
    sta     WSYNC
    sta     COLPM0
    lda     #$1e
    delay_scanline 29
    sta     COLPM0
    lda     #$98
    delay_scanline 2
    sta     COLPM0
    lda     #$4a
    delay_scanline 3
    sta     COLPM0
    lda     #$1e
    sta     WSYNC
    sta     COLPM0
    lda     #$4a
    delay_scanline 2
    sta     COLPM0
    lda     #$98
    sta     WSYNC
    sta     COLPM0
    lda     #$45
    delay_scanline 10
    sta     COLPM0
    lda     #$c4
    delay_scanline 5
    sta     COLPM0
    lda     #$45
    delay_scanline 2
    sta     COLPM0
    lda     #$ce
    delay_scanline 3
    sta     COLPM0
    lda     #$c4
    sta     WSYNC
    sta     COLPM0
    lda     #$ce
    delay_scanline 2
    sta     COLPM0
    lda     #$45
    sta     WSYNC
    sta     COLPM0

    pla
    tay
    pla
    rti

title_interrupt_2
    pha
    tya
    pha

    lda     #$a0
    delay_scanline 41
    sta     CHBASE
    lda     #$86
    sta     COLPF0
    lda     #$1e
    sta     COLPF1
    lda     #gate_color
    sta     COLPF2
    lda     #door_color
    sta     COLPF3
    lda     #$1e
    delay_scanline 3
    sta     COLPM0
    lda     #$98
    delay_scanline 2
    sta     COLPM0
    lda     #$4a
    delay_scanline 3
    sta     COLPM0
    lda     #$1e
    sta     WSYNC
    sta     COLPM0
    lda     #$4a
    delay_scanline 2
    sta     COLPM0
    lda     #$98
    sta     WSYNC
    sta     COLPM0
    lda     #$f8
    delay_scanline 3
    sta     CHBASE
    sty     COLPF2
    lda     #$0e
    sta     COLPF1

    pla
    tay
    pla
    rti

title_interrupt_3
    pha
    tya
    pha

    lda     #$30
    delay_scanline 41
    sta     CHBASE
    lda     #$86
    sta     COLPF0
    lda     #$1e
    sta     COLPF1
    lda     #gate_color
    sta     COLPF2
    lda     #door_color
    sta     COLPF3
    lda     #$c4
    delay_scanline 3
    sta     COLPM0
    lda     #$45
    delay_scanline 2
    sta     COLPM0
    lda     #$ce
    delay_scanline 3
    sta     COLPM0
    lda     #$c4
    sta     WSYNC
    sta     COLPM0
    lda     #$ce
    delay_scanline 2
    sta     COLPM0
    lda     #$45
    sta     WSYNC
    sta     COLPM0
    lda     #$f8
    delay_scanline 3
    sta     CHBASE
    sty     COLPF2
    lda     #$0e
    sta     COLPF1

    pla
    tay
    pla
    rti

ready_screen_interrupt subroutine
    pha
    tya
    pha
    txa
    pha

    lda     #$1c
    ldy     #$c9
    sta     WSYNC
    sta     COLPF1
    sty     COLPF2
    ldx     #52
    ldy     rider_color_offset
.delay_loop
    sta     WSYNC
    dex
    bpl     .delay_loop
    ldx     #16
.color_loop
    lda     rob_color_pattern + 1,y
    sta     WSYNC
    sta     COLPM0
    iny
    dex
    bne     .color_loop

    pla
    tax
    pla
    tay
    pla
    rti

intermission_2_interrupt subroutine
    pha
    tya
    pha

    delay_scanline 7
    ldy     #0
.cash_player_loop
    lda     $3e00,y
    sta     WSYNC
    sta     COLPM2
    iny
    cpy     #16
    bne     .cash_player_loop
    ldy     #$90
.phone_player_loop
    lda     $3e00,y
    sta     WSYNC
    sta     COLPM0
    iny
    cpy     #player_bottom_y + #20
    bne     .phone_player_loop

    pla
    tay
    pla
    rti

; Keypad handler.
keypad_interrupt subroutine
    ldy     keypad_cooldown
    bne     .wait_for_cooldown
    sta     last_key_pressed
    lda     #30
    sta     keypad_cooldown
.wait_for_cooldown
    jmp     ENDINT

