; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Fun little car intro for the current player

; Car animation constants.
car_frame_interval equ 7

; Data for the animation.
car_scroll          equ $3ff
animation_timer     equ $3fe
car_frame           equ $3fd
rider_pos           equ $3fc
rider_color_offset  equ $3fb

; Start the intro sequence for the current player
start_player_intro subroutine
    jsr     start_transition
    mwa     player_intro_display_list,sDLISTL
    mwa     player_intro_vblank,VVBLKD
    mwa     ready_screen_interrupt,VDSLST
    lda     #car_frame_interval
    sta     animation_timer
    ldy     #39
    lda     #0
.text_zone_clear_loop
    sta     $1100,y
    dey
    bpl     .text_zone_clear_loop
    mwa     get_ready_text,$fc
    mwa     $110c,$fe
    jsr     copy_text_full
    lda     $b0                     ; This is the active_player variable
    beq     .player_1               ; defined in game.s  DASM freaks out
    inc     $1113                   ; when you pre-use a zero page identifier.
    lda     #$45
    jmp     .set_player_color
.player_1
    lda     #$98
.set_player_color
    sta     sCOLPM0
    lda     #$f8
    sta     CHBASE
    lda     #$36
    sta     sCOLOR0
    lda     #$0e
    sta     sCOLOR1
    lda     #4
    sta     PRIOR
    lda     #28
    sta     rider_pos
    sta     HPOSP0
    ldy     #0
    tya
    sta     sCOLOR2
    sta     car_scroll
    sta     HSCROL
    sta     car_frame
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM0
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    sta     AUDCTL
.ram_clear_loop
    sta     $1400,y                     ; Clear player 0 and the car
    sta     $3400,y                     ; scroll area.
    sta     $3500,y
    sta     $3600,y
    sta     $3700,y
    sta     $3800,y
    sta     $3900,y
    dey
    bne     .ram_clear_loop
    jsr     build_car_display_list
    mwa     player_intro_display_list,$461
    ldy     #0
    jsr     draw_car
    ldy     #0
    jsr     draw_player_in_car
    jsr     complete_transition
    lda     #$b
    jsr     DOTUNE                  ; Start the engine noise.
    ; Fall through to the animation loop.

player_intro_loop subroutine
    jsr     wait_for_vb
    lda     rider_pos
    cmp     #$e0
    bne     .not_done
    lda     #$11                    ; Game priority.
    sta     PRIOR
    lda     new_game
    beq     .new_game
    jmp     start_game_from_intro
.new_game
    jmp     start_game
.not_done
    jsr     process_car
    jmp     player_intro_loop

; Build the car display list.  Factored out so the intermissions can
; use it.
build_car_display_list subroutine
    ldy     #16                         ; 32 mode e lines, but doubled.
    sty     $fd
    lda     #50
    sta     $fe
    lda     #$34
    sta     $ff
    ldy     #0
.loop
    lda     #$5e                        ; Mode 3 with load and hscroll.
    sta     $400,y                      ; Use each row twice.
    sta     $403,y
    lda     $fe
    iny
    sta     $400,y
    sta     $403,y
    lda     $ff
    iny
    sta     $400,y
    sta     $403,y
    iny
    iny
    iny
    iny
    lda     $fe
    clc
    adc     #80
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    dec     $fd
    bne     .loop
    lda     #$41
    sta     $460
    rts

; Move the car and the rider.
process_car subroutine
    ldy     car_scroll
    iny
    cpy     #16
    bne     .scroll_ok
    sty     $fd                     ; Coarse scroll, 16 rows.
    ldy     #0
.coarse_loop
    lda     $401,y
    sec
    sbc     #4
    sta     $401,y
    sta     $404,y
    lda     $402,y
    sbc     #0
    sta     $402,y
    sta     $405,y
    dec     $fd
    beq     .coarse_scroll_done
    tya
    clc
    adc     #6
    tay
    jmp     .coarse_loop
.coarse_scroll_done
    ldy     #0
.scroll_ok
    sty     HSCROL
    sty     car_scroll
    ldx     rider_pos
    inx
    stx     rider_pos
    stx     HPOSP0
    ldy     car_frame
    jsr     draw_player_in_car
    dec     animation_timer
    beq     .process_animation
    rts
.process_animation
    lda     #car_frame_interval
    sta     animation_timer
    lda     car_frame
    eor     #1
    sta     car_frame
    tay
    ; Fall through to draw code.

; Draw the car, frame number is expected in y.
draw_car subroutine
    cpy     #0                      ; Set up the source data.
    beq     .frame_1
    mwa     car_image_2,$fc
    jmp     .frame_ready
.frame_1
    mwa     car_image_1,$fc
.frame_ready
    mwa     $342f,$fe               ; Start address.
    lda     #15                     ; 15 rows to copy.
    sta     $fb
.outer_loop
    ldy     #6                      ; Copy 7 bytes.
.inner_loop
    lda     ($fc),y
    sta     ($fe),y
    dey
    bpl     .inner_loop
    dec     $fb
    beq     .done
    lda     $fc
    clc
    adc     #7
    sta     $fc
    bcc     .no_source_carry
    inc     $fd
.no_source_carry
    lda     $fe
    clc
    adc     #80
    sta     $fe
    bcc     .outer_loop
    inc     $ff
    jmp     .outer_loop
.done
    rts

; Draw the player in the car, vertical offset in y.
draw_player_in_car subroutine
    tya
    eor     #1
    ldx     $b0                     ; Active_player, again.  See above.
    beq     .player_1
    clc
    adc     #16
.player_1
    sta     rider_color_offset
    lda     rider_pos
    cmp     #$29
    bcc     .no_draw
    cmp     #$30
    bcc     .mask_left              ; Not completely on screen.
    cmp     #$c9
    bcc     .no_mask
    cmp     #$d0
    bcs     .zero_mask
    sec
    sbc     #$c9
    tax
    lda     rider_mask_right,x
    jmp     .mask_ready
.zero_mask
    lda     #0
    jmp     .mask_ready
.no_mask
    lda     #$ff
    jmp     .mask_ready
.mask_left
    lda     #$2f
    sec
    sbc     rider_pos
    tax
    lda     rider_mask_left,x
.mask_ready
    sta     $ff
    tya
    clc
    adc     #$83
    tax
    ldy     #15                     ; Copy 16 bytes.
.loop
    lda     $ff
    and     player_right_2,y
    sta     $1400,x
    dex
    dey
    bpl     .loop
.no_draw
    rts

player_intro_vblank
    lda     $fe
    pha
    lda     $ff
    pha
    jsr     TUNER
    pla
    sta     $ff
    pla
    sta     $fe
    lda     #0
    sta     ATRACT
    jmp     ENDINT

