; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; This is a somewhat modified version of code given to
; me to Bobby Clark, who also created all the music and
; sound effects.

; Originally intended for the Atari 8-bit computers, I've
; stripped out some stuff that isn't relevant to the 5200
; (PAL timing for example, there is no PAL 5200) and removed
; some parts of the subroutines that weren't relevant to my
; particular project.  Many routines saved registers on the
; stack, which was largely unneeded in my case, for example.

; A few things have been moved around in memory as well, but
; I expect this code will run on an A8 with minimum modification.

;	TUNE VARIABLES - $34 BYTES
TUNEAREA		EQU	$700
TUNESOFF		EQU	TUNEAREA+$0000				;1 BYTE  - FLAG FOR ALL TUNES OFF
TUNNUM			EQU	TUNEAREA+$0001				;1 BYTE  - CURRENT TUNE NUMBER BEING PROCESSED
TUNCHANNEL		EQU	TUNEAREA+$0002				;1 BYTE  - CONVERTED FOR INDEX INTO POKEY CHANNEL REGISTERS
PALCNT			EQU	TUNEAREA+$0003				;1 BYTE  - PAL index
TUNON			EQU	TUNEAREA+$0004				;4 BYTES - FLAG FOR TUNE PLAYING BY CHANNEL
NOTELO			EQU	TUNEAREA+$0008				;4 BYTES - INDEX INTO NOTE TABLE LOW
NOTEHI			EQU	TUNEAREA+$000C				;4 BYTES - INDEX INTO NOTE TABLE HI
CTLVOL			EQU	TUNEAREA+$0010				;4 BYTES - CONTROL / VOLUME VALUE BY CHANNEL
TUNINDEX		EQU	TUNEAREA+$0014				;4 BYTES - TUNE NUMBER BY CHANNEL
TUNFRM			EQU	TUNEAREA+$0018				;4 BYTES - NUMBER OF FRAMES (DURATION COUNT)
TUNPRIOR		EQU	TUNEAREA+$001C				;4 BYTES - TUNE PRIORITY BY CHANNEL
DURNLO			EQU	TUNEAREA+$0020				;4 BYTES - INDEX INTO DURATION TABLE LOW
DURNHI			EQU	TUNEAREA+$0024				;4 BYTES - INDEX INTO DURATION TABLE HI
DCYSTOR			EQU	TUNEAREA+$0028				;4 BYTES - FOR NOTE DECAY BY CHANNEL
FREQCNT			EQU	TUNEAREA+$002C				;4 BYTES - FOR NOTE DECAY BY CHANNEL
CTLSAV			EQU	TUNEAREA+$0030				;4 BYTES - TO SAVE THE CONTROL VALUE
MUTEMASK		EQU	TUNEAREA+$0036				;4 BYTES - FLAG FOR MUTEMASK

SOUNDZP equ $fe

;	SOUND -------------------------------------------------------------------------------------------------------------------->

;	TUNES - THESE ROUTINES HANDLE ALL OF THE SOUNDS

;	TURN OFF ALL SOUNDS
STOPTUN
	LDA #$00
	STA AUDC1
	STA AUDC2
	STA AUDC3
	STA AUDC4
	LDA #$01
	STA TUNESOFF
	RTS

;	TURN ON ALL SOUNDS
STARTTUN
	LDA #$00
	STA TUNESOFF
	RTS

;	THIS ROUTINE ERASES ALL TUNES
CLEARTUN
	LDX #$03
CTLOOP
	JSR ENDTUNE									;ERASE CURRENT TUNE
	DEX
	BPL CTLOOP
	RTS

;	ROUTINE TO KILL A PARTICULAR TUNE - IF IT IS RUNNING
;	INPUT: TUNE NUMBER IN A
KILLTUNE
	STA TUNNUM									;SAVE IT
	LDX #$03									;CHECK ALL CHANNELS
KTLOOP
	LDA TUNON,X									;SEE IF CHANNEL ON
	BEQ KTNEXT
	LDA TUNINDEX,X								;SEE IF HAS TUNE TO BE KILLED
	CMP TUNNUM
	BNE KTNEXT
	JSR ENDTUNE									;ERASE IT
KTNEXT
	DEX
	BPL KTLOOP
	RTS

;	THIS ROUTINE CLEARS OUT A TUNE CHANNEL
;	INPUT: X IS CHANNEL
ENDTUNE
	LDA #$00
	STA TUNON,X									;INDICATE CHANNEL CLEAR
	STA TUNINDEX,X								;CLEAR TUNE INDEX
	STA DCYSTOR,X
	STA FREQCNT,X
	RTS

;	THIS ROUTINE ENTERS A TUNE INTO ONE OF THE SOUND CHANNELS IF IT CAN
;	INPUT:  TUNE NUMBER IN A
DOTUNE
	STA TUNNUM									;SAVE IT
	LDY TUNNUM									;SEE IF WE CAN PUT IT IN
	LDX CHANNLTBL,Y								;GET WHAT CHANNEL TO TRY TO PUT IT IN
	LDA TUNON,X									;SEE IF CHANNEL OPEN
	BEQ DTDOIT
	LDA PRIRTYTBL,Y								;SEE IF WE CAN BUMP CHANNEL
	CMP TUNPRIOR,X
	BMI DTOUT
DTDOIT
	LDA TUNNUM
	TAY											;PUT TUNE IN Y
	STA TUNINDEX,X								;SET THE TUNE INDEX
	LDA #$00									;TURN TUNE OFF WHILE CHANGING IT
	STA TUNON,X
	LDA CNTVOLTBL,Y								;GET TUNE CONTROL / VOLUME
	STA CTLVOL,X
	STA CTLSAV,X								;USED TO RESTORE AFTER DECAY
	LDA NOTETBLLO,Y								;GET TUNE FREQUENCY LOW ADDRESS
	STA NOTELO,X
	LDA NOTETBLHI,Y								;GET TUNE FREQUENCY HIGH ADDRESS
	STA NOTEHI,X
	LDA DURNTBLLO,Y								;GET TUNE DURATION LOW ADDRESS
	STA DURNLO,X
	LDA DURNTBLHI,Y								;GET TUNE DURATION HIGH ADDRESS
	STA DURNHI,X
	LDA PRIRTYTBL,Y								;SET PRIORITY
	STA TUNPRIOR,X
	LDA #$01									;SET FREQ, CTL, AND VOL TO BE SET
	STA TUNFRM,X
	STA TUNON,X									;AND TURN THE TUNE ON!
DTOUT
	RTS

;  THIS ROUTINE IS CALLED EVERY VBLANK TO TAKE CARE OF TUNES
;  REGISTERS ARE NOT SAVED
TUNER
	LDX #$03									;FOUR TUNES CHANNELS, START WITH LAST
	LDA TUNESOFF
	BEQ TUNLOOP
	RTS
TUNLOOP
	TXA
	ASL
	STA TUNCHANNEL								;CHANNELS ARE OFFSET 0, 2, 4, 6 IN THE POKEY
	TAY
	LDA TUNON,X
	BNE TUNBODY
	STA AUDC1,Y									;CHANNEL OFF - MAKE SURE VOLUME OFF
	JMP TUNNEXT
TUNBODY
	DEC TUNFRM,X								;SEE IF WE'RE DONE WITH THIS SOUND
	BEQ TUNFRMFRQ								;YES - GET NEXT NOTE / DURATION
	DEC FREQCNT,X                               ;REDUCE THE NUMBER OF FRAMES UNTIL NEXT DECAY
	BEQ DEC_VOLUME
	JMP TUNNEXT                                 ;IF WE AREN'T AT ZERO YET, DON'T DECAY

DEC_VOLUME
	LDA DCYSTOR,X                               ;RESET THE DECAY FOR THE NEXT COUNT
	STA FREQCNT,X

	LDA CTLVOL,X                                ;IF VOLUME ALREADY 0 DO NOT DECREMENT
	AND #$0F
	BEQ TUNNEXT
	
	DEC CTLVOL,X                                ;DECREMENT THE VOLUME
	LDA CTLVOL,X
	LDY TUNCHANNEL
	AND MUTEMASK,X
	STA AUDC1,Y
	JMP TUNNEXT                                 ;GO TO NEXT CHANNEL
TUNFRMFRQ
	LDA DURNLO,X							;GET THE CURRENT DURATION
	STA SOUNDZP
	LDA DURNHI,X
	STA SOUNDZP+1
	LDY #$00
	LDA (SOUNDZP),Y
	BEQ TUNEND								;$00 IN DURATION MEANS TUNE IS OVER
	STA TUNFRM,X
	TAY
	LDA DECAYTBL,Y							;GET THE CURRENT DECAY VALUE INDEXED BY NOTE
	STA DCYSTOR,X							;STORE IT HERE TO REFRESH THE COUNTER FOR THE NEXT DECAY
	STA FREQCNT,X							;ALSO STORE IT HERE FOR TUNER
	LDY TUNCHANNEL
	LDA CTLSAV,X
	STA CTLVOL,X							;RESTORE THE ORIGINAL CONTROL AND VOLUME FOR NEXT NOTE
 	AND #$0f 								;REVENG - STATIC REMOVAL - ensure 0 write is skipped
 	BEQ SKIPWRITEZERO
	LDA CTLSAV,X

 

	STA AUDC1,Y
SKIPWRITEZERO
	LDA NOTELO,X							;GET THE CURRENT FREQUENCY
	STA SOUNDZP
	LDA NOTEHI,X
	STA SOUNDZP+1

 
	LDA #$ff
	STA MUTEMASK,X


	LDY #$00
	LDA (SOUNDZP),Y
	LDY TUNCHANNEL

 
        ; if you want to use 254 as a REST indicator, change the "CMP #0" to "CMP #254"
	CMP #0									;+ MUTEMASK KLUDGE
	BNE SKIPFLAGREST
	LDA #$f0
	STA MUTEMASK,X
	AND CTLSAV,X 
	STA AUDC1,Y 
	
	LDA #$00
SKIPFLAGREST
 

	STA AUDF1,Y
	INC NOTELO,X
	BNE TUNNEXTNOTE
	INC NOTEHI,X
TUNNEXTNOTE
	INC DURNLO,X
	BNE TUNNEXT
	INC DURNHI,X
TUNNEXT
	DEX
	CPX #$00
	BMI TUNEXIT
	JMP TUNLOOP
TUNEXIT

	RTS
	

TUNEND
	LDA NOTELO,X								;SEE IF WE SHOULD REPEAT
	STA SOUNDZP
	LDA NOTEHI,X
	STA SOUNDZP+1
	LDY #$00
	LDA (SOUNDZP),Y
	BMI TUNRESTART
	JSR ENDTUNE
	JMP TUNNEXT
TUNRESTART
	LDA TUNINDEX,X								;GET TUNE NUMBER
	TAY
	LDA CNTVOLTBL,Y								;GET TUNE CONTROL / VOLUME
	STA CTLVOL,X
	STA CTLSAV,X								;USED TO RESTORE AFTER DECAY
	LDA NOTETBLLO,Y								;GET TUNE FREQUENCY LOW ADDRESS
	STA NOTELO,X
	LDA NOTETBLHI,Y								;GET TUNE FREQUENCY HIGH ADDRESS
	STA NOTEHI,X
	LDA DURNTBLLO,Y								;GET TUNE DURATION LOW ADDRESS
	STA DURNLO,X
	LDA DURNTBLHI,Y								;GET TUNE DURATION HIGH ADDRESS
	STA DURNHI,X
	LDY TUNCHANNEL
	LDA CTLVOL,X
	STA AUDC1,Y									;STORE THE CONTROL / VOLUME IN THE CHANNEL
	LDA #$01									;SET FREQ, CTL, AND VOL TO BE SET
	STA TUNFRM,X
	JMP TUNNEXT

;	DATA FOR TUNES

;	CNTVOLTBL - HIGH NYBBLE FOR VOICE CONTROL (7654XXXX), AND LOW NYBBLE FOR VOLUME (XXXX3210)
CNTVOLTBL:
	.byte $00,$a6,$a6,$a1		 	;$A0 on channel 2 to silence channel
    .byte $a6,$a6,$86,$a4
	.byte $a4,$a4,$a4,$c4
	.byte $00,$a6,$a6,$a6
	.byte $a6,$a6,$00,$a5
	.byte $a5,$a5,$00,$a5
    .byte $a5,$a5,$00,$a5
    .byte $a5,$a5
	

;	PRIORITY TABLE
PRIRTYTBL:
	.byte $02,$02,$02,$02
	.byte $02,$02,$02,$04	
	.byte $02,$02,$02,$02
	.byte $02,$02,$02,$02
	.byte $02,$03,$02,$02
	.byte $02,$02,$02,$02
    .byte $02,$02,$02,$02
    .byte $02,$02
	
;	CHANNEL TABLE ($00 - $03 ARE VALID)
CHANNLTBL:
	.byte $00,$01,$02,$00			
	.byte $02,$02,$03,$01
	.byte $01,$01,$01,$01
	.byte $00,$01,$02,$01
	.byte $01,$01,$00,$01
	.byte $02,$03,$00,$01
    .byte $02,$03,$00,$01
    .byte $02,$03

;	DECAY TABLE - DEFINES DECAY COUNT INDEXED BY NOTE DURATION
DECAYTBL:
	.byte $01,$01,$01,$01,$02,$02,$02,$02
	.byte $02,$03,$03,$03,$03,$03,$03,$03
	.byte $04,$04,$04,$04,$04,$04,$04,$04
	.byte $05,$05,$05,$05,$05,$05,$05,$05
	.byte $05,$05,$06,$06,$06,$06,$06,$06
	.byte $06,$06,$06,$06,$07,$07,$07,$07
	.byte $07,$07,$07,$07,$07,$07,$07,$07
	.byte $07,$08,$08,$08,$08,$08,$08,$08

;	FREQUENCY TABLE
NOTETBLHI:
	.byte >(TUNE00),>(TUNE01),>(TUNE02),>(TUNE03)
	.byte >(TUNE04),>(TUNE05),>(TUNE06),>(TUNE07)
	.byte >(TUNE08),>(TUNE09),>(TUNE0A),>(TUNE0B)
	.byte >(TUNE0C),>(TUNE0D),>(TUNE0E),>(TUNE0F)
	.byte >(TUNE10),>(TUNE11),>(TUNE12),>(TUNE13)
    .byte >(TUNE14),>(TUNE15),>(TUNE16),>(TUNE17)
    .byte >(TUNE18),>(TUNE19),>(TUNE1A),>(TUNE1B)
    .byte >(TUNE1C),>(TUNE1D)

NOTETBLLO:
	.byte <(TUNE00),<(TUNE01),<(TUNE02),<(TUNE03)
	.byte <(TUNE04),<(TUNE05),<(TUNE06),<(TUNE07)
	.byte <(TUNE08),<(TUNE09),<(TUNE0A),<(TUNE0B)
	.byte <(TUNE0C),<(TUNE0D),<(TUNE0E),<(TUNE0F)
	.byte <(TUNE10),<(TUNE11),<(TUNE12),<(TUNE13)
    .byte <(TUNE14),<(TUNE15),<(TUNE16),<(TUNE17)
    .byte <(TUNE18),<(TUNE19),<(TUNE1A),<(TUNE1B)
    .byte <(TUNE1C),<(TUNE1D)

;	DURATION TABLE
DURNTBLHI:
	.byte >(DURN00),>(DURN01),>(DURN02),>(DURN03)
	.byte >(DURN04),>(DURN05),>(DURN06),>(DURN07)
	.byte >(DURN08),>(DURN09),>(DURN0A),>(DURN0B)
	.byte >(DURN0C),>(DURN0D),>(DURN0E),>(DURN0F)
	.byte >(DURN10),>(DURN11),>(DURN12),>(DURN13)
    .byte >(DURN14),>(DURN15),>(DURN16),>(DURN17)
    .byte >(DURN18),>(DURN19),>(DURN1A),>(DURN1B)
    .byte >(DURN1C),>(DURN1D)

DURNTBLLO:
	.byte <(DURN00),<(DURN01),<(DURN02),<(DURN03)
	.byte <(DURN04),<(DURN05),<(DURN06),<(DURN07)
	.byte <(DURN08),<(DURN09),<(DURN0A),<(DURN0B)
	.byte <(DURN0C),<(DURN0D),<(DURN0E),<(DURN0F)
	.byte <(DURN10),<(DURN11),<(DURN12),<(DURN13)
    .byte <(DURN14),<(DURN15),<(DURN16),<(DURN17)
    .byte <(DURN18),<(DURN19),<(DURN1A),<(DURN1B)
    .byte <(DURN1C),<(DURN1D)

;TEMPOS
T1	=	$03
T2      =       (T1*2)
T3      =       (T1*3)
T4      =       (T1*4)
T5      =       (T1*5)
T6      =       (T1*6)
T7      =       (T1*7)
T8      =       (T1*8)
T9      =       (T1*9)
TA      =       (T1*10)
TB      =       (T1*11)
TC      =       (T1*12)
TD      =       (T1*13)
TE      =       (T1*14)
TF      =       (T1*15)
T0      =       (T1*16)

 	
;level start music

TUNE00:

	.byte $c4,$c4,0,$32,$32,0,$fe,0,$32,0
	.byte $c4,$c4,$f8,$f8,$32,$32,$de,$de,$cf
	
	
	
	.byte 0
	
DURN00:
	.byte 6,6,12,6,6,12,6,18,6,18
	.byte 6,6,6,6,6,6,6,6,20
	
	

	
	.byte 0
	


TUNE01:

	
	
	.byte $25,$25,0,$19,$19,0,$13,0,$19,0
	.byte $25,$25,$1c,$1c,$19,$19,$12,$12,$10
	
	
	
	.byte 0
	

	
	

DURN01: 
	
	
	.byte 6,6,12,6,6,12,6,18,6,18
	.byte 6,6,6,6,6,6,6,6,20
	
	

	
	.byte 0
	
	
	
	
	
	
	


TUNE02:
	.byte 86,86,68,68,57,57,42,42
	.byte 45,42,45,50,57,64,68,76
	.byte 86,86,68,68,57,57,42,42,37
	
	
	
	.byte 0
	

DURN02:
	
	.byte 6,6,6,6,6,6,6,6
	.byte 6,6,6,6,6,6,6,6
	.byte 6,6,6,6,6,6,6,6,20

	
	.byte 0
	
	
	
	
;bg tune

TUNE03:

	.byte 33,42,37,45
	
	
	
	.byte -1
	
	
	
	

	
DURN03:
	
	.byte 6,6,6,6
	

	
	.byte 0
	
;eat dot 1

TUNE04:   
	.byte 33,30,26,25,23,22,23,0
	
	
DURN04:
	.byte 1,1,1,1,1,1,1,0

;eat dot 2
	
TUNE05:
	.byte 50,45,40,35,33,35,0
	
	
	
	
	
	
	
	


DURN05:
	.byte 1,1,1,1,1,1,1,0
	
	
	
	

;door close

	
TUNE06:
	.byte 3,0
	
	
	
	
DURN06:
    .byte 24,0
	
	
	
;money bag

TUNE07:
   .byte 31,31,42,42,25,25,31,25,21,21,23,25,28,28,0,-1
	

DURN07: 
	.byte 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,3,0
	
;moneybag pickup 

TUNE08:
	.byte 30,23,19,26,22,17,23,29,15,22,17,14
	.byte 19,15,12,17,14,11,15,12,9,0

;win level - previous sound flows into this one
TUNE09:
	.byte 53,47,42,53,47,42,40,53,47,42,40,53,0
	.byte 53,47,42,53,47,42,40,53,47,42,40,53,0
	.byte 53,47,42,53,47,42,40,53,47,42,40,53,0
	.byte 53,47,42,53,47,42,40,53,47,42,40,53,0
	.byte 53,47,42,53,47,42,40,53,47,42,40,53,0
	
	
	
	
DURN08:
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,10

DURN09:
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,10
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,10
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,10
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,10
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,10,0
	
	
;invalid door lock

TUNE0A:
	.byte 21,18,16,14,12,10,8,0
	


DURN0A:
	
	.byte 1,1,1,1,1,1,1,0
	
;car engine	

TUNE0B:   
	.byte 67,67,70,70,72,72,73,73,75,75
	.byte 76,76,78,78,81,81,82,82,85,85
	.byte 88,88,85,85,82,82,81,81,78,78
	.byte 76,76,75,75,73,73,72,72,70,70
	.byte -1
			

	
DURN0B:
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
	.byte 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
	.byte 0

;player death tune

;high - d#5 c5 a#4 f#4   d#4 c4 a#3 f#3   c4 a3 g3 d3  a#3
;low                     c4  a3  g3  d3   c3 a2 g2 d2  d2

;bass - two channels 
TUNE0C:
	.byte 0
	.byte $56,$dd,$cf,$c8,0
	.byte $b2,$c0,$a5,$96
	.byte $96,0
	
	
	
	
DURN0C:
	.byte 54
	.byte 6,6,6,30,6
	.byte 6,6,6,6
	.byte 60,0

TUNE0D:
	
	.byte 0
	.byte $0d,$0f,$11,$17,0
	.byte $1a,$1f,$23,$2f
	.byte $2f,0

DURN0D: 
	
	.byte 54
	.byte 6,6,6,30,6
	.byte 6,6,6,6
	.byte 60,0

;high part 

;high - d#5 c5 a#4 f#4   d#4 c#4 a#3 f#3  c4 a3 g3 d3  a#3
;low

TUNE0E:
	.byte 50,60,68,85,0
	.byte 102,121,136,172,0
	.byte 121,144,162,217
	.byte 136,0


DURN0E:
	
	.byte 6,12,6,24,6
	.byte 6,6,6,30,6
	.byte 6,6,6,6
	.byte 60,0

;extra life

TUNE0F:   
	.byte 40,40,40,40,40,40,40,40,40,0

	
DURN0F:
	.byte 10,10,10,10,10,10,10,10,0
	
	

;item pickup

	


TUNE10:
	.byte 10,11,12,14,15,16,18
	.byte 21,18,16,15,14,12,11,10,0
	
DURN10:
	.byte 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0

;cop trapped

TUNE11:
	.byte 35,35,35,40,45,35
	.byte 30,30,30,33,35,30
	.byte 0
	
DURN11:
	.byte 5,5,5,10,5,10
	.byte 5,5,5,10,5,10
	.byte 0
	
; Intermission 2
TUNE12: ;$8x guitar low order
	
	.byte $f8,$c8,$03,$c8,$b2,$72,$03,$72
	.byte $f8,$c8,$03,$c8,$b2,$72,$03,$72
	
	.byte 0,0
	
	.byte $b2,$72,$03,$72,$f8,$28,$f7
	
	.byte 0

	
	
	
	
	
	
	
	
DURN12:
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	
	.byte 60,60
	
	.byte 15,15,15,15,15,15,30
	
	.byte 0

	
	
;	TUNE 13 - $8X GUITAR HIGH ORDER
TUNE13:
	
	.byte $1d,$17,$28,$17,$1a,$16,$28,$16
	.byte $1d,$17,$28,$17,$1a,$16,$28,$16
	
	.byte 0,0
	
	.byte $1a,$16,$28,$16,$1d,$28,$3b
	
	.byte 0

	
	
	

DURN13: ;this table MUST be identical to DURN00:
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	
	.byte 60,60
	
	.byte 15,15,15,15,15,15,30
	
	.byte 0

	
	
;
;	TUNE 14
TUNE14:

	.byte 0,182,0,182,0,182,0,182
	.byte 0,182,0,182,0,182,0,182
	
	.byte 162,230,217,162,230,217
	
	.byte 0,182,0,182,0,204,217
	
	.byte 0

	
DURN14:
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	
	.byte 15,15,30,15,15,30
	
	.byte 15,15,15,15,15,15,30
	
	.byte 0

	
	

;
;	TUNE 15 - $Cx @15khz  bass
TUNE15:
	
	.byte 53,33,40,45,53,68
	.byte 72,45,40,60,50
	.byte 53,33,40,45,53,68
	.byte 72,45,40,60,91
	
	.byte 96,57,68,81
	.byte 91,68,53,45
	.byte 96,57,68,81
	.byte 91,68,53,45,53
	
	.byte 40,50,40,60,50,45,50,72
	.byte 68,91,136
	
	
	
	.byte 0

	
	
	
DURN15:
	
	.byte 15,8,15,7,8,7
	.byte 8,15,7,15,15
	.byte 15,8,15,7,8,7
	.byte 8,15,7,15,15
	
	.byte 5,5,5,15
	.byte 5,5,5,15
	.byte 5,5,5,15
	.byte 5,5,5,8,7
	
	.byte 8,7,8,7,8,7,8,7
	.byte 15,15,15
	
	
	
	.byte 0

; Intermission 1
TUNE16: ;$8x guitar low order
	
	.byte $72,$fe,$cf,$fe,$f8,$fe,$cf,$cf,$cf,$72
	.byte $a4,$c0,$f8,0
	
	
	
	;.byte $72,$cf,$f8,$cf,$72,$cf,$f8,$cf
	;.byte $72,$cf,$f8,$cf,$fe,$cf,$f8,$cf
	;.byte $fe,$cf,$f8,$cf,$fe,$cf,$f8,$cf
	;.byte $fe,$de,$cf,0
	
	.byte $72,$cf,$f8,$cf,$72,$cf,$f8,$cf
	.byte $49,$a5,$b2,0
	.byte $a4,$c0,$f8,$72,$f8,$72
	.byte $b2,$72,$f8,$c8,$72,$f8,$ea
	
	.byte 0

	
DURN16:
	
	.byte 8,7,8,22,15,8,7,8,22,15
	.byte 30,30,30,30
	
	;.byte 15,15,15,15,15,15,15,15
	;.byte 15,15,15,15,15,15,15,15
	;.byte 15,15,15,15,15,15,15,15
	;.byte 30,30,30,30
	
	.byte 15,15,15,15,15,15,15,15
	.byte 30,30,30,30
	.byte 30,30,15,15,15,15
	.byte 15,15,15,15,15,15,30
	
	.byte 0
	
	
	
	
;	TUNE 1 - $8X GUITAR HIGH ORDER
TUNE17:
	
	
	.byte $16,$14,$11,$14,$1d,$14,$11,$10,$11,$16
	.byte $21,$1f,$1d,0
	
	;.byte $16,$11,$1d,$11,$16,$11,$1d,$11
	;.byte $16,$11,$1d,$11,$14,$10,$1d,$10
	;.byte $14,$10,$1d,$10,$14,$10,$1d,$10
	;.byte $14,$12,$11,0
	
	.byte $16,$11,$1d,$11,$16,$11,$1d,$11
	.byte $1c,$23,$1a,0
	.byte $21,$1f,$1d,$16,$1d,$16
	.byte $1a,$16,$1d,$17,$16,$1d,$2c
	
	.byte 0
	
	
	

DURN17: ;this table MUST be identical to DURN00:
	
	.byte 8,7,8,22,15,8,7,8,22,15
	.byte 30,30,30,30
	
	;.byte 15,15,15,15,15,15,15,15
	;.byte 15,15,15,15,15,15,15,15
	;.byte 15,15,15,15,15,15,15,15
	;.byte 30,30,30,30
	
	.byte 15,15,15,15,15,15,15,15
	.byte 30,30,30,30
	.byte 30,30,15,15,15,15
	.byte 15,15,15,15,15,15,30
	
	.byte 0
	
;
;	TUNE 2 
TUNE18:
	
	
	.byte 53,45,40,45,68,45,40,37,40,50
	.byte 60,60,68,60,53,50,68,0
	
	.byte 81,76,72
	;.byte 68,50,53,60,68,50,53,60,68,81
	;.byte 68,50,53,60,68,53,60,68,91,81
	;.byte 76,53,60,68,76,53,60,68,53,45
	;.byte 37,37,35,33,50,45,40,81,76,72
	
	.byte 68,50,53,60,68,50,53,60,68,81
	.byte 40,40,45,50,30,26,25,60
	.byte 45,45,42,42,40,40,33,37,40
	.byte 40,40,50,45,50


	.byte 0
	
DURN18:
	
	.byte 8,7,8,22,15,8,7,8,22,15
	.byte 8,15,7,8,7,15,30,8
	
	.byte 7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,30,15,15,8,7,8,7,8,7
	
	.byte 15,15,15,15,8,15,15,7,8,7
	.byte 15,30,15,15,8,7,15,15
	.byte 15,15,15,15,8,15,7,15,15
	.byte 15,22,7,15,15
	
	.byte 0
	
	
	
	
	
	
	
	

;
;	TUNE 3 - $Cx @15khz  bass
TUNE19:
	.byte 60,53,50,53,81,53,50,45,50,68
	.byte 121,121,136,121,108,102,136,0
	
	.byte 102,91,76
	;.byte 68,50,53,60,68,50,53,60,68,81
	;.byte 68,50,53,60,68,53,60,68,91,81
	;.byte 76,53,60,68,76,53,60,68,53,45
	;.byte 37,37,35,33,50,45,40,81,76,72
	
	.byte 81,68,76,81,81,68,76,81,91,102
	.byte 64,64,81,60,0,0,0,81
	.byte 60,60,53,53,50,50,40,45,50
	.byte 50,50,68,53,68
	
	.byte 0
	
	
	
	
	
DURN19:
	
	.byte 8,7,8,22,15,8,7,8,22,15
	.byte 8,15,7,8,7,15,30,8
	
	.byte 7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,30,15,15,8,7,8,7,8,7
	
	.byte 15,15,15,15,8,15,15,7,8,7
	.byte 15,30,15,15,8,7,15,15
	.byte 15,15,15,15,8,15,7,15,15
	.byte 15,22,7,15,15
	
	.byte 0

; Intermission 3
TUNE1A: ;$8x guitar low order
	
	.byte 0
	
	.byte $72,$2f,$fe,0
	
	.byte $f8,$c8,$03,$c8,$f8,$c8,$03,$c8
	.byte $f8,$c8,$03,$c8,$f8,$c8,$fe,$cf
	.byte $a5,$f8,$c4,$f8,$03,$f8,$03,$f8
	
	.byte $a5,$c4,$03,$f8,$03,$f7
	
	.byte 0

	
DURN1A:
	
	.byte 120
	.byte 30,30,30,30
	
	
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,30,15,15,30
	
	
	
	.byte 0
	
	
	
	
;	TUNE 1 - $8X GUITAR HIGH ORDER
TUNE1B:
	
	
	.byte 0
	
	.byte $16,$15,$14,0
	
	.byte $1d,$17,$28,$17,$1d,$17,$28,$17
	.byte $1d,$17,$28,$17,$1d,$17,$14,$10
	.byte $23,$1d,$25,$1d,$28,$1d,$28,$1d
	
	.byte $23,$25,$28,$1d,$28,$3b
	
	.byte 0
	
	
	

DURN1B: ;this table MUST be identical to DURN00:
	
	.byte 120
	.byte 30,30,30,30
	
	
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,30,15,15,30
	
	
	
	.byte 0
	
;
;	TUNE 2 
TUNE1C:
	
	
	.byte 81,91,102,91,102,108,121,136,121,108,91,0
	.byte 81,81,91,81,72,68,60,91,0
	
	.byte 0,182,0,182,0,182,0,182
	.byte 0,182,0,182,0,91,91,91
	.byte 0,204,0,204,0,217,0,217
	.byte 162,162,172,172,182
	
	
	;.byte 68,50,53,60,68,50,53,60,68,81
	;.byte 68,50,53,60,68,53,60,68,91,81
	;.byte 76,53,60,68,76,53,60,68,53,45
	;.byte 37,37,35,33,50,45,40,81,76,72
	
	


	.byte 0
	
DURN1C:
	
	.byte 8,15,7,8,7,8,7,8,7,8,15,22
	.byte 8,15,7,8,7,15,15,30,15
	
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	.byte 15,15,15,15,15,15,15,15
	
	.byte 15,15,8,22,30
	
	
	
	.byte 0
	
	
	
	
	
	
	
	

;
;	TUNE 3 - $Cx @15khz  bass
TUNE1D:
	.byte 40,45,50,45,50,53,60,68,60,53,45,0
	.byte 40,40,45,40,35,33,30,45,91
	
	
	;.byte 68,50,53,60,68,50,53,60,68,81
	;.byte 68,50,53,60,68,53,60,68,91,81
	;.byte 76,53,60,68,76,53,60,68,53,45
	;.byte 37,37,35,33,50,45,40,81,76,72
	
	.byte 40,45,50,45,50,53,60,68,60,53,45,33,26
	.byte 40,45,50,45,50,53,50,45,45,53,45,53,45
	.byte 53,33,53,33,53,33,26,33,45,53,45,40,33
	.byte 68,68,60,53,68
	
	.byte 0
	
	
	
	
	
DURN1D:
	
	.byte 8,15,7,8,7,8,7,8,7,8,15,22
	.byte 8,15,7,8,7,15,15,30,15
	
	
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,15,15,15,8,15,15,7,8,7
	;.byte 15,30,15,15,8,7,8,7,8,7
	
	.byte 8,15,7,8,7,8,7,8,7,8,15,7,15
	.byte 8,15,7,8,7,8,7,15,8,7,8,7,15
	.byte 8,15,7,15,8,15,7,8,7,8,7,8,7
	.byte 15,15,7,15,22
	
	.byte 0
	
	
	
