; This file is part of Rob 'n' Banks.

; Rob 'n' Banks is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Rob 'n' Banks is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Rob 'n' Banks.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2022 Phaser Cat Games LLC

; Title screen logic and data

; Title screen data.
player_walk_pos         equ $20
player_walk_time        equ $21
player_walk_frame       equ $22
next_title_interrupt    equ $23
title_color_1           equ $25
title_color_2           equ $26
title_timer             equ $27

; Common memory area for persistent data.
game_mode       equ $2ff
start_stage     equ $2fe
last_p1_score   equ $2fb
last_p2_score   equ $2f8
high_score      equ $2f5
script_number   equ $2f2
new_game        equ $2f1
intermission    equ $2f0

; Game mode constants.
mode_one_player     equ 0
mode_two_player_one equ 1
mode_two_player_two equ 2
mode_intermission   equ 3
title_timer_length  equ 10

start_title_screen subroutine
    jsr     start_transition
    mwa     title_vblank,VVBLKD
    mwa     title_screen_display_list,sDLISTL
    mwa     title_logo_interrupt,VDSLST
    lda     game_mode
    and     #127                    ; Clear attract flag.
    sta     game_mode
    lda     #$11                    ; Enable fifth player.
    sta     PRIOR
    jsr     clear_sprites
    jsr     safe_clear_page_0
    mwa     title_interrupt_1,next_title_interrupt
    mva     #$36,title_color_1
    mva     #$1c,title_color_2
    mva     #title_timer_length,title_timer
    lda     #0
    ldy     #7
.blank_tile_loop
    sta     $3000,y
    dey
    bpl     .blank_tile_loop
    mwa     bonus_hat,$fc
    mwa     $3008,$fe
    ldy     #160
    jsr     copy_memory
    mwa     bonus_camera,$fc
    mwa     $30a8,$fe
    ldy     #160
    jsr     copy_memory
    lda     #$f8
    sta     CHBASE
    lda     #$20
    sta     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    inc     $ff
    jsr     clear_page
    jsr     hide_all_sprites
    mwa     score_line_string,$fc
    mwa     $2000,$fe
    jsr     copy_text
    lda     #3
    sta     $fe
    mwa     last_p1_score,$fc
    jsr     place_score
    lda     #18
    sta     $fe
    lda     #<high_score
    sta     $fc
    jsr     place_score
    lda     #34
    sta     $fe
    lda     #<last_p2_score
    sta     $fc
    jsr     place_score
    mwa     press_start_string,$fc
    mwa     $2305,$fe
    jsr     copy_text_full
    mwa     license_string,$fc
    mwa     $2350,$fe
    jsr     copy_text_full
    mwa     copyright_string,$fc
    mwa     $2371,$fe
    jsr     copy_text_full
    lda     #0
    sta     sCOLOR2
    sta     new_game
    sta     intermission
    lda     #$0e
    sta     sCOLOR1
    lda     #$36
    sta     sCOLOR0
    lda     #biff_color
    sta     sCOLPM1
    lda     #pork_chop_color
    sta     sCOLPM2
    lda     #bull_color
    sta     sCOLPM3
    lda     #buzz_color
    sta     sCOLOR3
    lda     #$ff
    sta     last_key_pressed
    lda     #4                      ; Switch to stick 1.
    sta     CONSOL
    lda     #$50
    sta     AUDCTL
    jsr     complete_transition
    ; Fall through to the attract loop.

attract_loop subroutine
    ldy     #30
    jsr     title_delay
    mwa     hero_string,$fc
    mwa     $2176,$fe
    jsr     copy_text
    ldy     #player_sprite_height - 1
.sprite_loop
    lda     player_right_1,y
    sta     $1478,y
    sta     $1490,y
    dey
    bpl     .sprite_loop
    lda     #$0e
    sta     sCOLPM0
    ldy     #30
    jsr     title_delay
    mwa     rob_string,$fc
    mwa     $21f5,$fe
    jsr     copy_text_full
    mwa     banks_string,$fc
    mwa     $226d,$fe
    jsr     copy_text_full
    lda     #$70
    sta     HPOSP0
    ldy     #120
    jsr     title_delay
    mwa     cop_string,$fc
    mwa     $2173,$fe
    jsr     copy_text
    lda     #0
    sta     HPOSP0
    ldy     #4
.hero_name_clear_loop
    sta     $21f5,y
    sta     $226d,y
    dey
    bpl     .hero_name_clear_loop
    ldy     #30
    jsr     title_delay
    mwa     biff_string,$fc
    mwa     $21ef,$fe
    jsr     copy_text_full
    mwa     buzz_string,$fc
    mwa     $2267,$fe
    jsr     copy_text_full
    mwa     bull_string,$fc
    mwa     $21f8,$fe,
    jsr     copy_text_full
    mwa     pork_chop_string,$fc
    mwa     $2270,$fe
    jsr     copy_text_full
    ldy     #player_sprite_height - 1
.cop_draw_loop
    lda     cop_right_1,y
    sta     $1390,y
    sta     $1578,y
    sta     $1690,y
    sta     $1778,y
    dey
    bpl     .cop_draw_loop
    lda     #$60
    sta     HPOSP1
    sta     HPOSM3
    lda     #$62
    sta     HPOSM2
    lda     #$64
    sta     HPOSM1
    lda     #$66
    sta     HPOSM0
    lda     #$84
    sta     HPOSP2
    sta     HPOSP3
    ldy     #180
    jsr     title_delay
    mwa     title_interrupt_2,next_title_interrupt
    lda     #0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM0
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
    ldy     #17
.erase_cop_names_loop
    sta     $2173,y
    sta     $21ef,y
    sta     $2267,y
    sta     $1478,y                 ; Erase Rob 'n' Banks too.
    sta     $1490,y
    dey
    bpl     .erase_cop_names_loop
    ldy     #player_sprite_height - 1
.player_draw_loop
    lda     player_right_1,y
    sta     $1487,y
    dey
    bpl     .player_draw_loop
    lda     #$98
    sta     sCOLPM0
    lda     #0
    sta     HPOSP0
    sta     player_walk_time
    sta     player_walk_frame
    ldy     #19
.cash_bag_draw_loop
    lda     title_cash_pattern_1,y
    sta     $2212,y
    lda     title_cash_pattern_2,y
    sta     $223a,y
    dey
    bpl     .cash_bag_draw_loop
.cash_bag_score_loop
    jsr     wait_for_vb
    jsr     check_pre_menu_keys
    jsr     animate_title_player
    cpx     #88
    bne     .not_bag_1
    lda     #0
    sta     $2212
    sta     $2213
    sta     $223a
    sta     $223b
    lda     #4
    sta     $21ca
    lda     #$15
    sta     $21cb
    lda     #$10
    sta     $21cc
    sta     $21cd
    jsr     pickup_delay
    jmp     .cash_bag_score_loop
.not_bag_1
    cpx     #112
    bne     .not_bag_2
    lda     #0
    sta     $2218
    sta     $2219
    sta     $2240
    sta     $2241
    ldx     #$11
    stx     $21cb
    dex
    stx     $21ce
    jsr     pickup_delay
    jmp     .cash_bag_score_loop
.not_bag_2
    cpx     #136
    bne     .not_bag_3
    lda     #0
    sta     $221e
    sta     $221f
    sta     $2246
    sta     $2247
    inc     $21cb
    jsr     pickup_delay
    jmp     .cash_bag_score_loop
.not_bag_3
    cpx     #160
    bne     .not_bag_4
    lda     #0
    sta     $2224
    sta     $2225
    sta     $224c
    sta     $224d
    lda     #$14
    sta     $21cb
    jsr     pickup_delay
    jmp     .cash_bag_score_loop
.not_bag_4
    cpx     #216
    beq     .start_bonus_display
    jmp     .cash_bag_score_loop
.start_bonus_display
    lda     #$45
    sta     sCOLPM0
    lda     #0
    sta     HPOSP0
    sta     player_walk_pos
    ldy     #4
.erase_score_loop
    sta     $21ca,y
    dey
    bpl     .erase_score_loop
    jsr     wait_for_vb
    mwa     title_interrupt_3,next_title_interrupt
    ldy     #28
.bonus_draw_loop
    lda     title_bonus_pattern_1,y
    sta     $220d,y
    lda     title_bonus_pattern_2,y
    sta     $2235,y
    dey
    bpl     .bonus_draw_loop
.bonus_score_loop
    jsr     wait_for_vb
    jsr     check_pre_menu_keys
    jsr     animate_title_player
    cpx     #68
    bne     .not_bonus_1
    lda     #0
    sta     $220d
    sta     $220e
    sta     $2235
    sta     $2236
    lda     #4
    sta     $21ca
    lda     #$12
    sta     $21cb
    lda     #$10
    sta     $21cc
    sta     $21cd
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_1
    cpx     #80
    bne     .not_bonus_2
    lda     #0
    sta     $2210
    sta     $2211
    sta     $2238
    sta     $2239
    inc     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_2
    cpx     #92
    bne     .not_bonus_3
    lda     #0
    sta     $2213
    sta     $2214
    sta     $223b
    sta     $223c
    lda     #$15
    sta     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_3
    cpx     #104
    bne     .not_bonus_4
    lda     #0
    sta     $2216
    sta     $2217
    sta     $223e
    sta     $223f
    lda     #$11
    sta     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_4
    cpx     #116
    bne     .not_bonus_5
    lda     #0
    sta     $2219
    sta     $221a
    sta     $2241
    sta     $2242
    lda     #$18
    sta     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_5
    cpx     #128
    bne     .not_bonus_6
    lda     #0
    sta     $221c
    sta     $221d
    sta     $2244
    sta     $2245
    ldx     #$11
    stx     $21cb
    dex
    stx     $21ce
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_6
    cpx     #140
    bne     .not_bonus_7
    lda     #0
    sta     $221f
    sta     $2220
    sta     $2247
    sta     $2248
    inc     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_7
    cpx     #152
    bne     .not_bonus_8
    lda     #0
    sta     $2222
    sta     $2223
    sta     $224a
    sta     $224b
    sta     $21ce
    lda     #$14
    sta     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_8
    cpx     #164
    bne     .not_bonus_9
    lda     #0
    sta     $2225
    sta     $2226
    sta     $224d
    sta     $224e
    dec     $21cb
    lda     #$10
    sta     $21ce
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_9
    cpx     #176
    bne     .not_bonus_10
    lda     #0
    sta     $2228
    sta     $2229
    sta     $2250
    sta     $2251
    lda     #$15
    sta     $21cb
    jsr     pickup_delay
    jmp     .bonus_score_loop
.not_bonus_10
    cpx     #216
    beq     .start_attract
    jmp     .bonus_score_loop
.start_attract
    lda     game_mode
    ora     #128                    ; Attract flag.
    sta     game_mode
    jmp     start_game

; Delay on the screen for y frames.
title_delay subroutine
    jsr     wait_for_vb
    jsr     check_pre_menu_keys
    dey
    bne     title_delay
    rts

; Check the start and license keys.
check_pre_menu_keys subroutine
    lda     last_key_pressed
    cmp     #KEYSTART
    beq     start_menu
    cmp     #KEYOCTOTHORPE
    beq     start_license
    rts

; Prepare the license screen.
start_license subroutine
    lda     #<license_display_list
    sta     sDLISTL
    lda     #>license_display_list
    sta     sDLISTH
    lda     #$ff
    sta     last_key_pressed
    lda     #0
    sta     GRACTL                  ; Disable sprites.
.loop
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #KEYOCTOTHORPE
    bne     .loop
    lda     #$ff
    sta     last_key_pressed
    lda     #<title_screen_display_list
    sta     sDLISTL
    lda     #>title_screen_display_list
    sta     sDLISTH
    lda     #3
    sta     GRACTL                  ; Reenable sprites.
    jsr     wait_for_vb
    rts

; Prepare the main menu.
start_menu subroutine
    lda     #0
    jsr     DOTUNE
    lda     #1
    jsr     DOTUNE
    lda     #2
    jsr     DOTUNE
    ldx     #$ff
    txs
    mwa     title_interrupt_3,next_title_interrupt
    ldy     #39
    lda     #0
    sta     HPOSP0
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    sta     HPOSM0
    sta     HPOSM1
    sta     HPOSM2
    sta     HPOSM3
.clear_loop
    sta     $2173,y                 ; Erase everything that could
    sta     $21ca,y                 ; be on the screen.
    sta     $21ef,y
    sta     $2267,y
    sta     $220d,y
    sta     $2235,y
    sta     $2348,y
    dey
    bpl     .clear_loop
    mwa     game_mode_string,$fc
    mwa     $2176,$fe
    jsr     copy_text_full
    jsr     place_mode_text
    mwa     start_stage_string,$fc
    mwa     $21ed,$fe
    jsr     copy_text_full
    jsr     place_start_stage
    lda     #$ff
    sta     last_key_pressed
.menu_loop
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #KEYASTERISK
    beq     .update_mode
    cmp     #KEYOCTOTHORPE
    beq     .update_stage
    cmp     #KEYSTART
    beq     .start_game
    cmp     #4
    bcc     .start_intermission
    jmp     .menu_loop
.start_intermission
    sta     intermission
    jmp     start_game
.update_mode
    lda     #$ff
    sta     last_key_pressed
    ldx     game_mode
    cpx     #mode_two_player_two
    beq     .roll_mode
    inx
    jmp     .store_mode
.roll_mode
    ldx     #0
.store_mode
    stx     game_mode
    jsr     place_mode_text
    jmp     .menu_loop
.update_stage
    lda     #$ff
    sta     last_key_pressed
    ldx     start_stage
    cpx     #max_bonus_object - 1
    beq     .roll_stage
    inx
    jmp     .store_stage
.roll_stage
    ldx     #0
.store_stage
    stx     start_stage
    jsr     place_start_stage
    jmp     .menu_loop
.start_game
    lda     #0
    sta     $b0                     ; Active player.
    jmp     start_player_intro

; Place a score into the top line.  Expects the score
; address in $fc-$fd and the destination address in
; $fe-$ff.
place_score subroutine
    lda     #0
    sta     $f0
    lda     #4
    sta     $f1
.loop
    ldy     $f0                     ; Byte count.
    lda     ($fc),y
    inc     $f0
    tax
    and     #$0f
    ora     #$10
    ldy     $f1                     ; Digit count.
    sta     ($fe),y
    dey
    bmi     .done
    txa
    lsr
    lsr
    lsr
    lsr
    ora     #$10
    sta     ($fe),y
    dey
    sty     $f1
    bpl     .loop
.done
    rts

; Place the appropriate text for the current game mode.
place_mode_text subroutine
    lda     game_mode
    beq     .one_player
    cmp     #mode_two_player_one
    beq     .two_player_1
    mwa     mode_two_player_two_string,$fc
    mwa     $2197,$fe
    jmp     copy_text_full
.one_player
    ldy     #27
    lda     #0
.erase_loop
    sta     $2197,y
    dey
    bpl     .erase_loop
    mwa     mode_one_player_string,$fc
    mwa     $219f,$fe
    jmp     copy_text_full
.two_player_1
    mwa     mode_two_player_one_string,$fc
    mwa     $2197,$fe
    jmp     copy_text_full

; Place the starting stage icon on the screen.
place_start_stage subroutine
    lda     start_stage
    tax
    asl                             ; Multiply by four to get tile.
    asl
    ora     bonus_image_mask,x      ; Apply the color mask.
    tay
    iny                             ; Add one to hop over the blank.
    sty     $221b                   ; Refer the case to Holmes.
    iny
    sty     $221c
    iny
    sty     $2243
    iny
    sty     $2244
    rts
    
; Walk the player from left to right.
; HPOS will be in X on return.
animate_title_player subroutine
    ldx     player_walk_time
    beq     .switch_frame
    dex
    txa
    jmp     .frame_done
.switch_frame
    lda     player_walk_frame
    eor     #1
    sta     player_walk_frame
    bne     .frame_2
    ldy     #player_sprite_height - 1
.frame_1_loop
    lda     player_right_1,y
    sta     $1487,y
    dey
    bpl     .frame_1_loop
    jmp     .frame_copy_done
.frame_2
    ldy     #player_sprite_height - 1
.frame_2_loop
    lda     player_right_2,y
    sta     $1487,y
    dey
    bpl     .frame_2_loop
.frame_copy_done
    lda     #15
.frame_done
    sta     player_walk_time
    ldx     player_walk_pos
    and     #1
    beq     .no_move
    inx
    stx     player_walk_pos
    stx     HPOSP0
.no_move
    rts

; Common delay code after attract bonus pickups.
pickup_delay subroutine
    ldy     #60
    lda     #0
    sta     player_walk_time
    jmp     title_delay

; Title screen vertical blank interrupt.
title_vblank subroutine
    lda     $ff
    pha
    lda     $fe
    pha
    jsr     TUNER
    pla
    sta     $fe
    pla
    sta     $ff
    lda     #0
    sta     ATRACT
    mwa     title_logo_interrupt,VDSLST
    lda     keypad_cooldown
    beq     .no_keypad_cooldown
    dec     keypad_cooldown
.no_keypad_cooldown
    dec     title_timer
    bne     .done
    mva     #title_timer_length,title_timer
    lda     title_color_1
    cmp     #$3c
    beq     .reset_title_colors
    inc     title_color_1
    dec     title_color_2
    jmp     ENDINT
.reset_title_colors
    mva     #$36,title_color_1
    mva     #$1c,title_color_2
.done
    jmp     ENDINT

